@@ -13,10 +13,11 @@ set -o nounset
 set -o pipefail
 
 module load daint-gpu cudatoolkit cdt-cuda
-module load /apps/daint/UES/jenkins/7.0.UP02/gpu/easybuild/modules/all/CMake/3.18.4
 module unload cray-libsci_acc
 module list
 
+export PATH=/project/cray/alazzaro/cmake/bin:$PATH
+
 set -o xtrace  # do not set earlier to avoid noise from module
 
 umask 0002  # make sure group members can access the data
