@@ -10,50 +10,76 @@
 ####################################################################################################
 
 import sys
-import os
-from glob import glob
 import re
 import json
 import argparse
+from pathlib import Path
 
 sys.path.append("../")
-from kernels.smm_acc_predict import descr_to_kernel  # noqa: E402
+from kernels.smm_acc import descr_to_kernel  # noqa: E402
 
 re_mnk = re.compile(r"tune_(\d+)x(\d+)x(\d+)")
 re_winner = re.compile(r"\nWINNER: \d+ (.+)\n")
 re_gflops = re.compile(r"# ([0-9.]+) GFlop/s")
 re_errors = re.compile(r"Number of errors: (\d+)\n")
 
+from dataclasses import dataclass  # noqa: E402
+
+
+@dataclass
+class awinner:
+    value: str = ""
+    missing: int = 0
+    incomplete: int = 0
+    n_errors: int = 0
+
+
+def tune_sort_key(path: Path):
+    try:
+        _, triple = path.name.split("_")
+        m, n, k = triple.split("x")
+    except ValueError:
+        return (0, 0, 0)  # sort non-matching dirs as they come at the beginning
+
+    return (int(m), int(n), int(k))
+
 
 # ===============================================================================
-def main():
+def main(tune_dir=Path(".")):
     winners = dict()
 
-    for d in glob("tune_*"):
-        if not os.path.isdir(d):
+    n_errors = 0
+    for dir in sorted(tune_dir.glob("tune_*"), key=tune_sort_key):
+        if not dir.is_dir():
             continue
 
-        for exe_fn in glob(d + "/tune_*main.c*"):
-            mnk = tuple([int(i) for i in re_mnk.search(exe_fn).groups()])
-            log_fn = exe_fn.replace("_main.cu", ".log").replace("_main.cpp", ".log")
-            error_code = 0
-            if not os.path.exists(log_fn):
-                winners[mnk] = "log missing: " + log_fn
+        for exe_fpath in sorted(dir.glob("tune_*main.c*")):
+            mnk = tuple(int(i) for i in re_mnk.search(exe_fpath.name).groups())
+            if mnk not in winners:
+                winners[mnk] = awinner()
+            log_fpath = exe_fpath.parent / exe_fpath.name.replace(
+                "_main.cu", ".log"
+            ).replace("_main.cpp", ".log")
+            if not log_fpath.exists():
+                winners[mnk] = awinner(value=f"log missing: {log_fpath}", missing=1)
                 print(
-                    "Missing log:",
-                    log_fn,
+                    "WARNING: Missing log:",
+                    log_fpath,
                     ", please re-run (cd tune_mxnxk; sbatch tune_mxnxk.job)",
                 )
-                error_code = 1
+                n_errors += 1
             else:
-                error_code += process_log(log_fn, mnk, winners)
+                n_errors += process_log(log_fpath, mnk, winners)
 
-            if error_code > 0:
-                print("Missing or incomplete logs")
-                return
+    if n_errors > 0:
+        print(f"WARNING: Found {int(n_errors)} issues, check above messages.")
 
     # Get kernel objects from list of strings
-    kernels = [descr_to_kernel(kernel_descr) for kernel_descr in winners.values()]
+    kernels = [
+        descr_to_kernel(kernel_descr.value)
+        for kernel_descr in winners.values()
+        if not (kernel_descr.missing or kernel_descr.incomplete)
+    ]
     kernels_dict = dict(zip([(k.m, k.n, k.k) for k in kernels], kernels))
     new_file = "../parameters/parameters.json"
     with open(new_file, "w") as f:
@@ -73,17 +99,15 @@ def main():
 
 
 # ===============================================================================
-def process_log(log_fn, mnk, winners):
-    print("Reading: " + log_fn)
-
-    with open(log_fn) as f:
-        content = f.read()
+def process_log(log_fn: Path, mnk, winners):
+    print(f"Reading: {log_fn}")
 
+    content = log_fn.read_text()
     m = re_errors.search(content)
     if not m:
-        winners[mnk] = "log incomplete: " + log_fn
+        winners[mnk].incomplete += 1
         print(
-            "Found incomplete log:",
+            "WARNING: Found incomplete log:",
             log_fn,
             ", please re-run (cd tune_mxnxk; sbatch tune_mxnxk.job)",
         )
@@ -91,21 +115,22 @@ def process_log(log_fn, mnk, winners):
 
     n_errors = int(m.group(1))
     if n_errors != 0:
-        winners[mnk] = "errors: " + log_fn
-        return 0
+        winners[mnk].n_errors += n_errors
+        return 1
 
     old_gflops = 0.0
-    if mnk in winners.keys():
-        m = re_gflops.search(winners[mnk])
-        if not m:
-            return 0
+    m = re_gflops.search(winners[mnk].value)
+    if m:
         old_gflops = float(m.group(1))
 
-    new_winner = re_winner.search(content).group(1).strip().replace("GFlops", "GFlop/s")
+    new_winner = re_winner.search(content)
+    if not new_winner:
+        return 0
+    new_winner = new_winner.group(1).strip().replace("GFlops", "GFlop/s")
     new_gflops = float(re_gflops.search(new_winner).group(1))
 
     if new_gflops > old_gflops:
-        winners[mnk] = new_winner
+        winners[mnk].value = new_winner
     return 0
 
 
