@@ -11,11 +11,12 @@
 
 import sys
 import os
+import stat
 import json
 import math
-from glob import glob
 from itertools import product
 import argparse
+from pathlib import Path
 
 sys.path.append("../")
 from kernels.smm_acc_predict import (  # noqa: E402
@@ -28,27 +29,27 @@ from kernels.smm_acc_predict import (  # noqa: E402
 
 # ===============================================================================
 def main(
-    param_fn,
+    param_fn: Path,
     compiler,
     cpus_per_task,
     max_num_nodes,
     blocksizes,
     blocks_from_param_file,
-    tune_dir,
+    tune_dir: Path,
 ):
 
     # Read existing parameters
     assert (
-        os.path.basename(param_fn) in gpu_architectures.keys()
-    ), "Cannot find GPU architecture for file " + os.path.basename(param_fn)
-    arch_code = gpu_architectures[os.path.basename(param_fn)]
-    with open("../kernels/gpu_properties.json") as f:
-        gpu_properties = json.load(f)[arch_code]
-    with open("../kernels/autotuning_properties.json") as f:
-        autotuning_properties = json.load(f)
-    with open(param_fn) as f:
-        all_kernels = [params_dict_to_kernel(**params) for params in json.load(f)]
-    print("Reading parameters from %s" % param_fn)
+        param_fn.name in gpu_architectures.keys()
+    ), f"Cannot find GPU architecture for file {param_fn.name}"
+    arch_code = gpu_architectures[param_fn.name]
+    with open("../kernels/gpu_properties.json") as fhandle:
+        gpu_properties = json.load(fhandle)[arch_code]
+    with open("../kernels/autotuning_properties.json") as fhandle:
+        autotuning_properties = json.load(fhandle)
+    with param_fn.open("r") as fhandle:
+        all_kernels = [params_dict_to_kernel(**params) for params in json.load(fhandle)]
+    print(f"Reading parameters from {param_fn}")
     autotuned_kernels = [k for k in all_kernels if k.autotuned]
     predicted_kernels = [k for k in all_kernels if not k.autotuned]
     print(
@@ -62,14 +63,14 @@ def main(
             all_kernels_ref = [
                 params_dict_to_kernel(**params) for params in json.load(f)
             ]
-        print("Reading parameters to autotune from %s" % blocksizes)
+        print(f"Reading parameters to autotune from {blocksizes}")
         triples = [(k.m, k.n, k.k) for k in all_kernels_ref if k.autotuned]
     else:
         assert len(set(blocksizes)) == len(blocksizes)
         blocksizes.sort()
         # Get (m, n, k) triplets to be autotuned
         triples = combinations(*blocksizes)
-    print("Requested to autotune %d triplets" % len(triples))
+    print(f"Requested to autotune {len(triples)} triplets")
 
     for (m, n, k) in triples:
         existing = [kern for kern in autotuned_kernels if kern.can_handle(m, n, k)]
@@ -80,11 +81,11 @@ def main(
             )
             continue
 
-        outdir = os.path.join(tune_dir, "tune_%dx%dx%d/" % (m, n, k))
-        if os.path.exists(outdir):
-            print("Directory %s exists already, skipping." % outdir)
+        outdir = tune_dir / f"tune_{int(m)}x{int(n)}x{int(k)}"
+        if outdir.exists():
+            print(f"Directory {outdir} exists already, skipping.")
             continue
-        os.mkdir(outdir)
+        outdir.mkdir()
         gen_benchmark(outdir, gpu_properties, autotuning_properties, compiler, m, n, k)
         gen_jobfile(outdir, compiler, m, n, k, cpus_per_task, max_num_nodes)
         gen_makefile(outdir, compiler, arch_code)
@@ -108,13 +109,13 @@ def format_params(params):
     ]
     for k in order:
         if k in params.keys():
-            output.append("%s=%d" % (k, params[k]))
+            output.append(f"{k}={int(params[k])}")
 
     for k in params.keys():
         if k not in order:
-            output.append("%s=%d" % (k, params[k]))
+            output.append(f"{k}={int(params[k])}")
 
-    return "(" + ", ".join(output) + ")"
+    return f"({', '.join(output)})"
 
 
 def get_file_extension_from_compiler(compiler):
@@ -132,9 +133,9 @@ def gen_benchmark(outdir, gpu_properties, autotuning_properties, compiler, m, n,
 
     # Get the kernel algorithms compatible with the given size:
     compatible_kernels = [
-        kernel_algorithm[kernclass]
-        for kernclass in kernel_algorithm.keys()
-        if compatible_mnk(kernclass, m, n, k)
+        kernclass
+        for classname, kernclass in kernel_algorithm.items()
+        if compatible_mnk(classname, m, n, k)
     ]
 
     # Get the parameter sets to measure for this (m,n,k)
@@ -147,19 +148,19 @@ def gen_benchmark(outdir, gpu_properties, autotuning_properties, compiler, m, n,
 
         for p in params:
             kern = kernclass(**p, source="autotuning_candidate", perf=0)
-            includes.append("../../kernels/" + kern.include)
+            includes.append(f"../../kernels/{kern.include}")
             launcher_codes.append(kern.launcher_code(compiler))
-            launchers.append("launch_" + kern.name)
+            launchers.append(f"launch_{kern.name}")
             kernel_descr.append(kernclass.__name__ + format_params(p))
 
-    print("Found %d parameter sets for %dx%dx%d" % (len(launchers), m, n, k))
+    print(f"Found {len(launchers)} parameter sets for {int(m)}x{int(n)}x{int(k)}")
     if len(launchers) == 0:
         return
 
     # Compose the "include" line of the benchmark code
     incl_output = '#include "../../kernels/smm_acc_common.h"\n'
     for i in set(includes):
-        incl_output += '#include "%s"\n' % i
+        incl_output += f'#include "{i}"\n'
     incl_output += "\n\n"
 
     # Compose the benchmark code
@@ -168,9 +169,8 @@ def gen_benchmark(outdir, gpu_properties, autotuning_properties, compiler, m, n,
     # - each executable is made of n_obj_files object files
     # - each object file is made up of launchers_per_obj launchers
     # - each launcher launches 1 GPU kernel with a certain set of kernel parameters
-    # the hipcc compiler is very slow -> make a larger number of smaller executables
-    max_launchers_per_exe = 10000 if compiler == "nvcc" else 100
-    launchers_per_obj = 100 if compiler == "nvcc" else 10
+    max_launchers_per_exe = 10000
+    launchers_per_obj = 100
     n_exe_files = int(len(launcher_codes) / max_launchers_per_exe) + 1
     launchers_per_exe = int(len(launcher_codes) / n_exe_files) + 1
 
@@ -180,29 +180,24 @@ def gen_benchmark(outdir, gpu_properties, autotuning_properties, compiler, m, n,
         chunk_b = min((i + 1) * launchers_per_exe, len(launcher_codes))
         n_obj_files = math.ceil((chunk_b - chunk_a) / launchers_per_obj)
 
+        if n_obj_files == 0:
+            continue
+        jdigits = int(math.log10(n_obj_files)) + 1
+
         # Compose source code for each object file
         for j in range(n_obj_files):
             a = chunk_a + j * launchers_per_obj
             b = min(chunk_a + (j + 1) * launchers_per_obj, chunk_b)
             output = incl_output
             output += "\n\n".join(launcher_codes[a:b])
-            fn = outdir + "/tune_%dx%dx%d_exe%d_part%d%s" % (
-                m,
-                n,
-                k,
-                i,
-                j,
-                file_extension,
-            )
+            fn = outdir / f"tune_{m}x{n}x{k}_exe{i}_part{j:0{jdigits}}{file_extension}"
             writefile(fn, output)
 
         # Compose source code for "main" of executable file
         output = '#include "../../libsmm_acc_benchmark.h"\n\n'
         for j in range(chunk_b - chunk_a):
             output += (
-                "int "
-                + launchers[chunk_a + j]
-                + "(const int *param_stack, int stack_size, "
+                f"int {launchers[chunk_a + j]}(const int *param_stack, int stack_size, "
             )
             if compiler == "nvcc":
                 output += "cudaStream_t stream, "
@@ -220,19 +215,19 @@ def gen_benchmark(outdir, gpu_properties, autotuning_properties, compiler, m, n,
                 indent
                 + "cudaError_t err = cudaDeviceSetSharedMemConfig(cudaSharedMemBankSizeEightByte);\n"
             )
-            output += indent + "if(err != cudaSuccess) return(-1);\n"
+            output += f"{indent}if(err != cudaSuccess) return(-1);\n"
         else:  # i.e. compiler = hipcc
             output += (
                 indent
                 + "hipError_t err = hipDeviceSetSharedMemConfig(hipSharedMemBankSizeEightByte);\n"
             )
-            output += indent + "if(err != hipSuccess) return(-1);\n"
-        output += indent + "libsmm_acc_benchmark_t* handle;\n"
-        output += indent + "KernelLauncher launchers[%d];\n" % (chunk_b - chunk_a)
-        output += indent + "char *kernel_descr[%d];\n" % (chunk_b - chunk_a)
+            output += f"{indent}if(err != hipSuccess) return(-1);\n"
+        output += f"{indent}libsmm_acc_benchmark_t* handle;\n"
+        output += f"{indent}KernelLauncher launchers[{int(chunk_b - chunk_a)}];\n"
+        output += f"{indent}char *kernel_descr[{int(chunk_b - chunk_a)}];\n"
 
         for j in range(chunk_b - chunk_a):
-            output += indent + "launchers[%d]    = %s;\n" % (j, launchers[chunk_a + j])
+            output += f"{indent}launchers[{int(j)}]    = {launchers[chunk_a + j]};\n"
             output += indent + 'kernel_descr[%d] = (char *) "%s";\n' % (
                 j,
                 kernel_descr[chunk_a + j],
@@ -247,11 +242,13 @@ def gen_benchmark(outdir, gpu_properties, autotuning_properties, compiler, m, n,
             + "int result = libsmm_acc_benchmark(handle, %d, %d, %d, %d, launchers, kernel_descr);\n"
             % (m, n, k, chunk_b - chunk_a)
         )
-        output += indent + "libsmm_acc_benchmark_finalize(handle);\n"
-        output += indent + "return result;"
+        output += f"{indent}libsmm_acc_benchmark_finalize(handle);\n"
+        output += f"{indent}return result;"
         output += "}\n"
 
-        fn = outdir + "/tune_%dx%dx%d_exe%d_main%s" % (m, n, k, i, file_extension)
+        fn = (
+            outdir / f"tune_{int(m)}x{int(n)}x{int(k)}_exe{int(i)}_main{file_extension}"
+        )
         writefile(fn, output)
 
 
@@ -260,47 +257,51 @@ def gen_jobfile(outdir, compiler, m, n, k, cpus_per_task=12, max_num_nodes=0):
 
     file_extension = get_file_extension_from_compiler(compiler)
 
-    t = "/tune_%dx%dx%d" % (m, n, k)
-    all_exe_src = [
-        os.path.basename(fn) for fn in glob(outdir + t + "_*_main" + file_extension)
-    ]
-    all_exe = sorted([fn.replace("_main" + file_extension, "") for fn in all_exe_src])
+    tprefix = f"tune_{int(m)}x{int(n)}x{int(k)}"
+    all_exe_src = [fn.name for fn in outdir.glob(f"{tprefix}_*_main{file_extension}")]
+    all_exe = sorted([fn.replace(f"_main{file_extension}", "") for fn in all_exe_src])
     if max_num_nodes > 0:
         num_nodes = min(len(all_exe), max_num_nodes)
     else:
         num_nodes = len(all_exe)
     if num_nodes < 3:
-        time = "1:30:00"
+        time = "4:00:00"
     else:
         time = "0:30:00"
 
-    output = "#!/bin/bash -l\n"
-    output += "#SBATCH --nodes=%d\n" % num_nodes
-    output += "#SBATCH --ntasks-per-core=1\n"
-    output += "#SBATCH --ntasks-per-node=1\n"
-    output += "#SBATCH --cpus-per-task=" + "%d\n" % cpus_per_task
-    output += "#SBATCH --time=%s\n" % time
-    output += "#SBATCH --partition=bp11\n"
-    output += "\n"
-    output += "source ${MODULESHOME}/init/sh; module use /global/opt/modulefiles;\n"
-    output += "module unload PrgEnv-cray\n"
-    output += "module load PrgEnv-gnu\n"
+    output = f"""\
+#!/bin/bash -l
+#SBATCH --nodes={int(num_nodes)}
+#SBATCH --exclusive
+#SBATCH --ntasks-per-core=1
+#SBATCH --ntasks-per-node=1
+#SBATCH --cpus-per-task={int(cpus_per_task)}
+#SBATCH --time={time}
+#SBATCH --partition=bardpeak
+
+source ${{MODULESHOME}}/init/sh; module use /global/opt/modulefiles;
+module unload PrgEnv-cray
+module load PrgEnv-gnu
+"""
     if compiler == "nvcc":
         output += "module load cudatoolkit/8.0.61_2.4.9-6.0.7.0_17.1__g899857c\n"
     else:  # i.e. compiler = hipcc
         output += "module load rocm/5.1.0; module load craype-accel-amd-gfx90a;\n"
-    output += "export ROCR_VISIBLE_DEVICES=0\n"
-    output += "module list\n"
-    output += "cd $SLURM_SUBMIT_DIR \n"
-    output += "\n"
-    output += "date\n"
+
+    output += """\
+export ROCR_VISIBLE_DEVICES=4 # GPU corresponding to Numa node 0
+module list
+cd $SLURM_SUBMIT_DIR
+
+date
+"""
 
     # Compilation
     num_nodes_busy = 0
     for exe in all_exe:
         output += (
-            "srun --nodes=1 --bcast=/tmp/${USER} --ntasks=1 --ntasks-per-node=1 --cpus-per-task=%d make -j %d %s &\n"
-            % (cpus_per_task, 2 * cpus_per_task, exe)
+            f"srun --nodes=1 --bcast=/tmp/${{USER}} --ntasks=1 --ntasks-per-node=1"
+            f" --cpus-per-task={cpus_per_task} --exact make -j {cpus_per_task} {exe} &\n"
         )
         num_nodes_busy += 1
         if num_nodes_busy == num_nodes:
@@ -314,11 +315,8 @@ def gen_jobfile(outdir, compiler, m, n, k, cpus_per_task=12, max_num_nodes=0):
     # Execution
     for exe in all_exe:
         output += (
-            "srun --nodes=1 --bcast=/tmp/${USER} --ntasks=1 --ntasks-per-node=1 --cpus-per-task=1 ./"
-            + exe
-            + " >"
-            + exe
-            + ".log 2>&1 & \n"
+            f"srun --nodes=1 --bcast=/tmp/${{USER}} --ntasks=1 --ntasks-per-node=1"
+            f" --cpus-per-task=1 --exact ./{exe} > {exe}.log 2>&1 & \n"
         )
         num_nodes_busy += 1
         if num_nodes_busy == num_nodes:
@@ -331,15 +329,11 @@ def gen_jobfile(outdir, compiler, m, n, k, cpus_per_task=12, max_num_nodes=0):
 
     # Winner
     output += "echo Over all winner:\n"
-    output += (
-        "grep WINNER ."
-        + t
-        + '_exe*.log  |  sort -n --field-separator="#" -k 2 | tail -n 1\n'
-    )
+    output += f"grep WINNER {tprefix}_exe*.log  |  sort -n --field-separator='#' -k 2 | tail -n 1\n"
     output += "\n"
     output += "#EOF\n"
 
-    fn = outdir + t + ".job"
+    fn = outdir / f"{tprefix}.job"
     writefile(fn, output)
 
 
@@ -350,16 +344,16 @@ def gen_makefile(outdir, compiler, arch):
 
     # header
     output = ".SECONDARY:\n"
-    output += "vpath %" + file_extension + "../\n\n"
+    output += f"vpath %{file_extension}../\n\n"
     output += ".PHONY: do_nothing build_all \n\n"
     output += "do_nothing:\n\n"
 
     # target "build_all"
     all_exe_src = sorted(
-        [os.path.basename(fn) for fn in glob(outdir + "/tune_*_main" + file_extension)]
+        [fn.name for fn in outdir.glob(f"tune_*_main{file_extension}")]
     )
-    build_targets = [fn.replace("_main" + file_extension, "") for fn in all_exe_src]
-    output += "build_all: " + " ".join(build_targets) + "\n\n"
+    build_targets = [fn.replace(f"_main{file_extension}", "") for fn in all_exe_src]
+    output += f"build_all: {' '.join(build_targets)}\n\n"
 
     # compilation rule for helper-files: libsmm_acc_benchmark, acc_cuda/hip
     output += "libsmm_acc_benchmark.o : ../../libsmm_acc_benchmark.cpp\n"
@@ -380,65 +374,58 @@ def gen_makefile(outdir, compiler, arch):
         output += "\thipcc -O3 -D__TUNING -D__HIP -w -c -o $@ $<\n\n"
 
     # compilation rule for kernel files
-    headers = " ".join(["../" + fn for fn in glob("../kernels/*.h")])
-    output += "%.o : %" + file_extension + " " + headers + "\n"
+    headers = " ".join([f"../{fn}" for fn in Path("../kernels").glob("*.h")])
+    output += f"%.o : %{file_extension} {headers}\n"
     if compiler == "nvcc":
-        output += "\tnvcc -O3 -D__TUNING -D__CUDA -arch=" + str(arch) + " -w -c $<\n\n"
+        output += f"	nvcc -O3 -D__TUNING -D__CUDA -arch={str(arch)} -w -c $<\n\n"
     else:
         output += "\thipcc -O3 -D__TUNING -D__HIP -w -c $<\n\n"
 
     # compilation rule for autotuning executables
     for exe_src in all_exe_src:
-        absparts = sorted(
-            glob(outdir + "/" + exe_src.replace("_main" + file_extension, "_part*"))
+        parts = sorted(
+            [
+                fn.name
+                for fn in outdir.glob(
+                    exe_src.replace(f"_main{file_extension}", "_part*")
+                )
+            ]
         )
-        parts = [os.path.basename(fn) for fn in absparts]
         deps = [exe_src, "libsmm_acc_benchmark.cpp", "acc.cpp"] + parts
         deps_obj = " ".join(
             [fn.replace(".cu", ".o").replace(".cpp", ".o") for fn in deps]
         )
-        exe = exe_src.replace("_main" + file_extension, "")
-        output += exe + " : " + deps_obj + "\n"
+        exe = exe_src.replace(f"_main{file_extension}", "")
+        output += f"{exe} : {deps_obj}\n"
         if compiler == "nvcc":
             output += (
-                "\tnvcc -O3 -D__CUDA -arch="
-                + str(arch)
-                + " -w -o $@ $^ -lcuda -lnvrtc\n\n"
+                f"	nvcc -O3 -D__CUDA -arch={str(arch)} -w -o $@ $^ -lcuda -lnvrtc\n\n"
             )
         else:
             rocm_path = os.getenv("ROCM_PATH", "/opt/rocm")
             output += f"\thipcc -O3 -D__HIP -w -o $@ $^ {rocm_path}/hip/lib/libamdhip64.so\n\n"
 
     # write Makefile
-    writefile(outdir + "/Makefile", output)
+    writefile(outdir / "Makefile", output)
 
 
 # ===============================================================================
-def gen_collect(outdir, triples):
+def gen_collect(outdir: Path, triples):
     output = "#!/bin/bash\n"
-    for (m, n, k) in triples:
-        t = "/tune_%dx%dx%d" % (m, n, k)
-        output += (
-            "grep WINNER ."
-            + t
-            + '_exe*.log  |  sort -n --field-separator="#" -k 2 | tail -n 1\n'
-        )
+    for m, n, k in triples:
+        output += f"grep WINNER tune_{int(m)}x{int(n)}x{int(k)}_exe*.log  |  sort -n --field-separator='#' -k 2 | tail -n 1\n"
     output += "#EOF\n"
-    fn = outdir + "/collect_winners.sh"
+    fn = outdir / "collect_winners.sh"
     writefile(fn, output)
-    os.system("chmod +x " + fn)
+    fn.chmod(fn.stat().st_mode | stat.S_IEXEC)
 
 
 # ===============================================================================
-def writefile(fn, content):
-    if os.path.exists(fn):
-        with open(fn, "r") as f:
-            old_content = f.read()
-        if old_content == content:
-            return
+def writefile(fn: Path, content):
+    if fn.exists() and fn.read_text() == content:
+        return
 
-    with open(fn, "w") as f:
-        f.write(content)
+    fn.write_text(content)
 
 
 # ===============================================================================
@@ -458,19 +445,12 @@ if __name__ == "__main__":
         """,
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
     )
-    parser.add_argument(
-        "-d",
-        "--dir",
-        metavar="tune_directory",
-        default=".",
-        type=str,
-        help="Path from which to read already-existing tune-folders and write new tune-folders",
-    )
     parser.add_argument(
         "-p",
         "--params",
         metavar="parameters_GPU.json",
         default="../parameters/parameters_P100.json",
+        type=Path,
         help="Parameter file that this autotuning should extend (pick the right GPU)",
     )
     parser.add_argument(
@@ -514,9 +494,7 @@ if __name__ == "__main__":
     valid_compilers = ["nvcc", "hipcc"]
     assert (
         args.compiler in valid_compilers
-    ), "Compiler chosen ({}) is not valid, please choose among: {}".format(
-        args.compiler, valid_compilers
-    )
+    ), f"Compiler chosen ({args.compiler}) is not valid, please choose among: {valid_compilers}"
 
     # ==========
     # Blocksizes from parameter file or as list of integers
@@ -535,5 +513,5 @@ if __name__ == "__main__":
         args.nodes,
         args.blocksizes,
         blocksizes_from_param_file,
-        args.dir,
+        Path("."),
     )
