@@ -1,6 +1,6 @@
 /*----------------------------------------------------------------------------*/
 /*  CP2K: A general program to perform molecular dynamics simulations         */
-/*  Copyright 2000-2021 CP2K developers group <https://cp2k.org>              */
+/*  Copyright 2000-2022 CP2K developers group <https://cp2k.org>              */
 /*                                                                            */
 /*  SPDX-License-Identifier: BSD-3-Clause                                     */
 /*----------------------------------------------------------------------------*/
@@ -42,6 +42,9 @@ void dgemm_simplified(dgemm_params *const m) {
     abort();
 
 #if defined(__LIBXSMM)
+#if LIBXSMM_VERSION2(1, 17) >=                                                 \
+        LIBXSMM_VERSION2(LIBXSMM_VERSION_MAJOR, LIBXSMM_VERSION_MINOR) &&      \
+    (2079 > LIBXSMM_VERSION_PATCH)
   if (m->use_libxsmm && m->op2 == 'N') {
     /* we are in row major but xsmm is in column major */
     m->prefetch = LIBXSMM_PREFETCH_AUTO;
@@ -62,6 +65,7 @@ void dgemm_simplified(dgemm_params *const m) {
     }
   }
 #endif
+#endif
 
 #if defined(__MKL)
   // fall back to mkl
@@ -107,6 +111,9 @@ void batched_dgemm_simplified(dgemm_params *const m, const int batch_size) {
   assert(batch_size > 0);
 
 #if defined(__LIBXSMM)
+#if LIBXSMM_VERSION2(1, 17) >=                                                 \
+        LIBXSMM_VERSION2(LIBXSMM_VERSION_MAJOR, LIBXSMM_VERSION_MINOR) &&      \
+    (2079 > LIBXSMM_VERSION_PATCH)
   if (m->use_libxsmm && m->op2 == 'N') {
     /* we are in row major but xsmm is in column major */
     m->prefetch = LIBXSMM_PREFETCH_AUTO;
@@ -131,6 +138,7 @@ void batched_dgemm_simplified(dgemm_params *const m, const int batch_size) {
     }
   }
 #endif
+#endif
 
 #if defined(__MKL)
   // fall back to mkl
