@@ -1,15 +1,16 @@
 /*----------------------------------------------------------------------------*/
 /*  CP2K: A general program to perform molecular dynamics simulations         */
-/*  Copyright 2000-2021 CP2K developers group <https://cp2k.org>              */
+/*  Copyright 2000-2022 CP2K developers group <https://cp2k.org>              */
 /*                                                                            */
 /*  SPDX-License-Identifier: BSD-3-Clause                                     */
 /*----------------------------------------------------------------------------*/
 
-#ifdef __GRID_CUDA
+#include "../../offload/offload_runtime.h"
+
+#if defined(__OFFLOAD) && !defined(__NO_OFFLOAD_GRID)
 
 #include <algorithm>
 #include <assert.h>
-#include <cuda.h>
 #include <limits.h>
 #include <math.h>
 #include <stdio.h>
@@ -117,7 +118,7 @@ typedef struct {
  * \brief Shared memory representation of a task.
  * \author Ole Schuett
  ******************************************************************************/
-typedef struct : grid_gpu_task_struct {
+typedef struct smem_task_struct : grid_gpu_task_struct {
   // angular momentum range of actual collocate / integrate operation
   int la_max;
   int lb_max;
@@ -166,10 +167,7 @@ static void init_constant_memory() {
       }
     }
   }
-  cudaError_t error =
-      cudaMemcpyToSymbol(coset_inv, &coset_inv_host, sizeof(coset_inv_host), 0,
-                         cudaMemcpyHostToDevice);
-  assert(error == cudaSuccess);
+  offloadMemcpyToSymbol(coset_inv, &coset_inv_host, sizeof(coset_inv_host));
 
   // Binomial coefficient
   double binomial_coef_host[19][19];
@@ -179,10 +177,8 @@ static void init_constant_memory() {
       binomial_coef_host[n][k] = fac(n) / fac(k) / fac(n - k);
     }
   }
-  error =
-      cudaMemcpyToSymbol(binomial_coef, &binomial_coef_host,
-                         sizeof(binomial_coef_host), 0, cudaMemcpyHostToDevice);
-  assert(error == cudaSuccess);
+  offloadMemcpyToSymbol(binomial_coef, &binomial_coef_host,
+                        sizeof(binomial_coef_host));
 
   initialized = true;
 }
@@ -327,8 +323,7 @@ general_cxyz_to_grid(const kernel_params *params, const smem_task *task,
  *        (x-a)**lxa (x-b)**lxb -> sum_{ls} alpha(ls,lxa,lxb,1)*(x-p)**ls
  * \author Ole Schuett
  ******************************************************************************/
-__device__ static void compute_alpha(const kernel_params *params,
-                                     const smem_task *task, double *alpha) {
+__device__ static void compute_alpha(const smem_task *task, double *alpha) {
   // strides for accessing alpha
   const int s3 = (task->lp + 1);
   const int s2 = (task->la_max + 1) * s3;
@@ -364,8 +359,7 @@ __device__ static void compute_alpha(const kernel_params *params,
  * \brief Transforms coefficients C_ab into C_xyz.
  * \author Ole Schuett
  ******************************************************************************/
-__device__ static void cab_to_cxyz(const kernel_params *params,
-                                   const smem_task *task, const double *alpha,
+__device__ static void cab_to_cxyz(const smem_task *task, const double *alpha,
                                    GRID_CONST_WHEN_COLLOCATE double *cab,
                                    GRID_CONST_WHEN_INTEGRATE double *cxyz) {
 
@@ -468,7 +462,7 @@ __device__ static void load_task(const kernel_params *params, smem_task *task) {
     const int itask = params->first_task + blockIdx.x;
     const grid_gpu_task *src_task = &params->tasks[itask];
     grid_gpu_task *dest_task = task; // Upcast to base struct.
-    for (int i = threadIdx.x; i < sizeof(grid_gpu_task); i += blockDim.x) {
+    for (int i = threadIdx.x; i < (int)sizeof(grid_gpu_task); i += blockDim.x) {
       ((char *)dest_task)[i] = ((const char *)src_task)[i];
     }
   }
@@ -500,5 +494,5 @@ __device__ static void load_task(const kernel_params *params, smem_task *task) {
   __syncthreads(); // because of concurrent writes to task
 }
 
-#endif // __GRID_CUDA
+#endif // defined(__OFFLOAD) && !defined(__NO_OFFLOAD_GRID)
 // EOF
