@@ -1,13 +1,14 @@
 /*----------------------------------------------------------------------------*/
 /*  CP2K: A general program to perform molecular dynamics simulations         */
-/*  Copyright 2000-2021 CP2K developers group <https://cp2k.org>              */
+/*  Copyright 2000-2022 CP2K developers group <https://cp2k.org>              */
 /*                                                                            */
 /*  SPDX-License-Identifier: BSD-3-Clause                                     */
 /*----------------------------------------------------------------------------*/
 #ifndef GRID_GPU_TASK_LIST_H
 #define GRID_GPU_TASK_LIST_H
 
-#ifdef __GRID_CUDA
+#include "../../offload/offload_runtime.h"
+#if defined(__OFFLOAD) && !defined(__NO_OFFLOAD_GRID)
 
 #ifdef __cplusplus
 extern "C" {
@@ -16,7 +17,6 @@ extern "C" {
 #include "../../offload/offload_buffer.h"
 #include "../common/grid_basis_set.h"
 #include "../common/grid_constants.h"
-#include <cuda_runtime.h>
 #include <stdbool.h>
 
 /*******************************************************************************
@@ -111,8 +111,8 @@ typedef struct {
   int nblocks;
   grid_gpu_layout *layouts;
   int *tasks_per_level;
-  cudaStream_t *level_streams;
-  cudaStream_t main_stream;
+  offloadStream_t *level_streams;
+  offloadStream_t main_stream;
   int lmax;
   int stats[2][20]; // [has_border_mask][lp]
   // device pointers
@@ -172,7 +172,7 @@ void grid_gpu_integrate_task_list(const grid_gpu_task_list *task_list,
 }
 #endif
 
-#endif // __GRID_CUDA
+#endif // defined(__OFFLOAD) && !defined(__NO_OFFLOAD_GRID)
 #endif
 
 // EOF
