@@ -1,6 +1,6 @@
 /*----------------------------------------------------------------------------*/
 /*  CP2K: A general program to perform molecular dynamics simulations         */
-/*  Copyright 2000-2021 CP2K developers group <https://cp2k.org>              */
+/*  Copyright 2000-2022 CP2K developers group <https://cp2k.org>              */
 /*                                                                            */
 /*  SPDX-License-Identifier: BSD-3-Clause                                     */
 /*----------------------------------------------------------------------------*/
@@ -10,6 +10,7 @@
 #include <stdbool.h>
 
 #include "../offload/offload_buffer.h"
+#include "../offload/offload_runtime.h"
 #include "common/grid_basis_set.h"
 #include "common/grid_constants.h"
 #include "cpu/grid_cpu_task_list.h"
@@ -27,10 +28,11 @@ typedef struct {
   int (*npts_local)[3];
   grid_ref_task_list *ref;
   grid_cpu_task_list *cpu;
-#ifdef __GRID_CUDA
-  grid_gpu_task_list *gpu;
-#elif defined(__GRID_HIP)
+#if defined(__OFFLOAD) && !defined(__NO_OFFLOAD_GRID)
   grid_gpu_task_list *gpu;
+#endif
+#if defined(__OFFLOAD_HIP) && !defined(__NO_OFFLOAD_GRID)
+  grid_hip_task_list *hip;
 #endif
   // more backends to be added here
 } grid_task_list;
