@@ -1,6 +1,6 @@
 /*----------------------------------------------------------------------------*/
 /*  CP2K: A general program to perform molecular dynamics simulations         */
-/*  Copyright 2000-2021 CP2K developers group <https://cp2k.org>              */
+/*  Copyright 2000-2022 CP2K developers group <https://cp2k.org>              */
 /*                                                                            */
 /*  SPDX-License-Identifier: BSD-3-Clause                                     */
 /*----------------------------------------------------------------------------*/
@@ -13,12 +13,20 @@
 #include "common/grid_library.h"
 #include "grid_replay.h"
 
-void mpi_sum_func(long *number, int mpi_comm) {
+/*******************************************************************************
+ * \brief Standin for mpi_sum, passed to grid_library_print_stats.
+ * \author Ole Schuett
+ ******************************************************************************/
+static void mpi_sum_func(long *number, int mpi_comm) {
   *number += 0; // Nothing todo without MPI, pretend arguments are used anyways.
   mpi_comm += 0;
 }
 
-void print_func(char *message, int output_unit) {
+/*******************************************************************************
+ * \brief Wrapper for printf, passed to grid_library_print_stats.
+ * \author Ole Schuett
+ ******************************************************************************/
+static void print_func(char *message, int output_unit) {
   output_unit += 0; // Pretend argument is used.
   printf("%s", message);
 }
@@ -62,7 +70,7 @@ int main(int argc, char *argv[]) {
     return 1;
   }
 
-  offload_set_device_id(0);
+  offload_set_chosen_device(0);
   grid_library_init();
 
   int errors = 0;
