@@ -1,6 +1,6 @@
 /*----------------------------------------------------------------------------*/
 /*  CP2K: A general program to perform molecular dynamics simulations         */
-/*  Copyright 2000-2021 CP2K developers group <https://cp2k.org>              */
+/*  Copyright 2000-2022 CP2K developers group <https://cp2k.org>              */
 /*                                                                            */
 /*  SPDX-License-Identifier: GPL-2.0-or-later                                 */
 /*----------------------------------------------------------------------------*/
@@ -156,6 +156,20 @@ void cp2k_get_forces(force_env_t force_env, double *force, int n_el);
  ******************************************************************************/
 void cp2k_get_potential_energy(force_env_t force_env, double *e_pot);
 
+/*******************************************************************************
+ * \brief Get the size of the cell
+ * \param force_env the force environment
+ * \param cell Array containing the cell
+ ******************************************************************************/
+void cp2k_get_cell(force_env_t force_env, const double *cell);
+
+/*******************************************************************************
+ * \brief Get the size of the qmmm cell
+ * \param force_env the force environment
+ * \param cell Array containing the qmmm cell
+ ******************************************************************************/
+void cp2k_get_qmmm_cell(force_env_t force_env, const double *cell);
+
 /*******************************************************************************
  * \brief Calculate energy and forces of the system
  * \param force_env the force environment
