@@ -1,6 +1,8 @@
 #!/usr/bin/env python3
 
-import re, sys, os
+import os
+import re
+import sys
 from os import path
 
 # pre-compiled regular expressions
@@ -45,7 +47,7 @@ def is_fortran_program(fn):
     tail = f.read()
     f.close()
     m = re_program.search(tail.lower())
-    return m != None
+    return m is not None
 
 
 # ============================================================================
@@ -54,7 +56,7 @@ def has_main_function(fn):
     content = f.read()
     f.close()
     m = re_main.search(content)
-    return m != None
+    return m is not None
 
 
 # ===============================================================================
