@@ -3,7 +3,7 @@
 # author: Ole Schuett
 
 from pathlib import Path
-from typing import Any, Optional
+from typing import Any
 import argparse
 import io
 
@@ -21,23 +21,32 @@ def main() -> None:
             f.write(toolchain_full() + production(version))
 
         with OutputFile(f"Dockerfile.test_generic_{version}", args.check) as f:
-            f.write(toolchain_full(generic=True) + regtest(version))
+            f.write(toolchain_full(target_cpu="generic") + regtest(version))
 
         with OutputFile(f"Dockerfile.prod_generic_{version}", args.check) as f:
-            f.write(toolchain_full(generic=True) + production(version))
+            f.write(toolchain_full(target_cpu="generic") + production(version))
 
     with OutputFile(f"Dockerfile.test_openmpi-psmp", args.check) as f:
         # Also testing --with-gcc=install here, see github.com/cp2k/cp2k/issues/2062 .
-        f.write(toolchain_full(mpi_mode="openmpi", gcc="install") + regtest("psmp"))
+        f.write(toolchain_full(mpi_mode="openmpi", with_gcc="install"))
+        f.write(regtest("psmp"))
 
     with OutputFile(f"Dockerfile.test_intel-psmp", args.check) as f:
-        f.write(toolchain_intel())
-        f.write("\n# TODO: Remove --mpiranks=1, see github.com/cp2k/cp2k/issues/2103\n")
-        f.write(regtest("psmp", testopts="--mpiranks=1"))
+        f.write(toolchain_intel() + regtest("psmp"))
+
+    with OutputFile(f"Dockerfile.test_nvhpc", args.check) as f:
+        f.write(toolchain_nvhpc())
 
     with OutputFile(f"Dockerfile.test_minimal", args.check) as f:
         f.write(toolchain_full() + regtest("sdbg", "minimal"))
 
+    with OutputFile(f"Dockerfile.test_cmake", args.check) as f:
+        f.write(toolchain_full() + install_cp2k_cmake())
+
+    for version in "ssmp", "psmp":
+        with OutputFile(f"Dockerfile.test_asan-{version}", args.check) as f:
+            f.write(toolchain_full() + regtest(version, "local_asan"))
+
     for version in "sdbg", "pdbg":
         with OutputFile(f"Dockerfile.test_coverage-{version}", args.check) as f:
             f.write(toolchain_full() + coverage(version))
@@ -52,6 +61,16 @@ def main() -> None:
         f.write(toolchain_ubuntu_nompi(base_image="i386/debian:11", libvori=False))
         f.write(regtest("ssmp"))
 
+    with OutputFile("Dockerfile.test_arm64-psmp", args.check) as f:
+        f.write(
+            toolchain_full(
+                base_image="arm64v8/ubuntu:22.04",
+                with_libxsmm="no",
+                with_libtorch="no",
+            )
+        )
+        f.write(regtest("psmp"))
+
     with OutputFile(f"Dockerfile.test_performance", args.check) as f:
         f.write(toolchain_full() + performance())
 
@@ -358,17 +377,52 @@ COPY ./tools/regtesting ./tools/regtesting
 """
 
 
+# ======================================================================================
+def install_cp2k_cmake() -> str:
+    # TODO: This is a draft and does not yet actually work.
+
+    return rf"""
+
+COPY ./tools/build_utils/fypp /bin/fypp
+# temporary solution we build dbcsr without using the cloned repo. It will eventually be moved inside the toolchain
+COPY ./tools/docker/scripts/install_dbcsr.sh ./scripts/
+RUN  ./scripts/install_dbcsr.sh && rm -rf ./build
+
+# Install CP2K using CMake.
+
+WORKDIR /opt/cp2k
+COPY ./src ./src
+COPY ./exts ./exts
+COPY ./tools/build_utils ./tools/build_utils
+COPY ./cmake ./cmake
+COPY ./CMakeLists.txt .
+COPY ./cp2k.pc.in .
+WORKDIR ./build
+RUN /bin/bash -c " \
+    echo 'Compiling cp2k...' && \
+    ls /opt/cp2k-toolchain/install/scalapack-2.2.1 && \
+    ls /opt/cp2k-toolchain/install/fftw-3.3.10/lib && \
+    source /opt/cp2k-toolchain/install/setup && \
+    export PKG_CONFIG_PATH=/opt/cp2k-toolchain/install/libxsmm-1.17/lib:/opt/cp2k-toolchain/install/openblas-0.3.21/lib/pkgconfig:/opt/cp2k-toolchain/install/libxc-6.0.0/lib/pkgconfig:/opt/cp2k-toolchain/install/fftw-3.3.10/lib/pkgconfig:/opt/cp2k-toolchain/install/libint-v2.6.0-cp2k-lmax-5/lib/pkgconfig:/opt/cp2k-toolchain/install/plumed-2.8.0/lib/pkgconfig:/opt/cp2k-toolchain/install/superlu_dist-6.1.0/lib/pkgconfig && \
+    cmake -DCP2K_USE_COSMA=OFF -DCP2K_USE_LIBXSMM=NO -DSCALAPACK_ROOT=/opt/cp2k-toolchain/install/scalapack-2.2.1 -DCP2K_BLAS_VENDOR=OpenBLAS -DLibXC_ROOT=/opt/cp2k-toolchain/install/libxc-6.0.0 -DLibint2_ROOT=/opt/cp2k-toolchain/install/libint-v2.6.0-cp2k-lmax-5 -DDBCSR_ROOT=/opt/cp2k-toolchain/install/DBCSR-2.4.1 -DCP2K_USE_SPGLIB=ON -DCP2K_USE_LIBINT2=NO -DCP2K_USE_LIBXC=ON -DLibSPG_ROOT=/opt/cp2k-toolchain/install/spglib-1.16.2 .. && \
+    make -j"
+COPY ./data ./data
+COPY ./tests ./tests
+COPY ./tools/regtesting ./tools/regtesting
+
+RUN echo "\nSummary: Compilation works fine.\nStatus: OK\n"
+
+#EOF
+"""
+
+
 # ======================================================================================
 def toolchain_full(
-    base_image: str = "ubuntu:22.04",
-    mpi_mode: str = "mpich",
-    gcc: str = "system",
-    generic: bool = False,
+    base_image: str = "ubuntu:22.04", with_gcc: str = "system", **kwargs: str
 ) -> str:
-    args = dict(install_all="", mpi_mode=mpi_mode, with_gcc=gcc)
-    if generic:
-        args["generic"] = ""
-    return f"\nFROM {base_image}\n\n" + install_toolchain(base_image=base_image, **args)
+    return f"\nFROM {base_image}\n\n" + install_toolchain(
+        base_image=base_image, install_all="", with_gcc=with_gcc, **kwargs
+    )
 
 
 # ======================================================================================
@@ -416,15 +470,13 @@ RUN ln -sf /usr/bin/gcc-{gcc_version}      /usr/local/bin/gcc  && \
 
 # ======================================================================================
 def toolchain_intel() -> str:
-    # See https://github.com/cp2k/cp2k/issues/1936
     return rf"""
-FROM intel/oneapi-hpckit:2021.4-devel-ubuntu18.04
+FROM intel/oneapi-hpckit:2022.2-devel-ubuntu20.04
+
+# Without this cp2k segfaults right after startup.
+# See https://github.com/cp2k/cp2k/issues/1936
+ENV I_MPI_FABRICS='shm'
 
-ENV PATH=/opt/intel/oneapi/compiler/2021.4.0/linux/bin/intel64:/opt/intel/oneapi/mpi/2021.4.0/bin:${{PATH}}
-ENV LD_LIBRARY_PATH=/opt/intel/oneapi/mpi/2021.4.0/lib/release:/opt/intel/oneapi/mpi/2021.4.0/lib:/opt/intel/oneapi/mpi/2021.4.0/libfabric/lib:/opt/intel/oneapi/mkl/2021.4.0/lib/intel64:/opt/intel/oneapi/compiler/2021.4.0/linux/compiler/lib/intel64_lin:${{LD_LIBRARY_PATH}}
-ENV MKLROOT=/opt/intel/oneapi/mkl/2021.4.0
-ENV I_MPI_ROOT=/opt/intel/oneapi/mpi/2021.4.0
-ENV FI_PROVIDER_PATH='/opt/intel/oneapi/mpi/2021.4.0/libfabric/lib/prov'
 """ + install_toolchain(
         base_image="ubuntu",
         with_intel="",
@@ -437,15 +489,67 @@ ENV FI_PROVIDER_PATH='/opt/intel/oneapi/mpi/2021.4.0/libfabric/lib/prov'
     )
 
 
+# ======================================================================================
+def toolchain_nvhpc() -> str:
+    return rf"""
+FROM ubuntu:22.04
+
+# Install Ubuntu packages.
+RUN apt-get update -qq && apt-get install -qq --no-install-recommends \
+    apt-transport-https \
+    ca-certificates \
+    dirmngr \
+    gnupg2 \
+    libopenblas-dev \
+    make \
+    nano \
+    python3 \
+    wget \
+   && rm -rf /var/lib/apt/lists/*
+
+RUN apt-key adv --fetch-keys https://developer.download.nvidia.com/hpc-sdk/ubuntu/DEB-GPG-KEY-NVIDIA-HPC-SDK
+RUN echo 'deb https://developer.download.nvidia.com/hpc-sdk/ubuntu/amd64 /' > /etc/apt/sources.list.d/nvhpc.list
+
+# Install NVIDIA's HPC SDK but only keep the compilers to reduce Docker image size.
+RUN apt-get update -qq && \
+    apt-get install -qq --no-install-recommends nvhpc-22-11 && \
+    rm -rf /var/lib/apt/lists/* && \
+    rm -rf /opt/nvidia/hpc_sdk/Linux_x86_64/22.11/math_libs && \
+    rm -rf /opt/nvidia/hpc_sdk/Linux_x86_64/22.11/comm_libs && \
+    rm -rf /opt/nvidia/hpc_sdk/Linux_x86_64/22.11/profilers && \
+    rm -rf /opt/nvidia/hpc_sdk/Linux_x86_64/22.11/cuda
+
+ENV PATH ${{PATH}}:/opt/nvidia/hpc_sdk/Linux_x86_64/22.11/compilers/bin
+
+# Install CP2K using Linux-x86-64-nvhpc.ssmp.
+WORKDIR /opt/cp2k
+COPY ./Makefile .
+COPY ./src ./src
+COPY ./exts ./exts
+COPY ./data ./data
+COPY ./tests ./tests
+COPY ./tools/build_utils ./tools/build_utils
+COPY ./tools/regtesting ./tools/regtesting
+COPY ./arch/Linux-x86-64-nvhpc.ssmp /opt/cp2k/arch/
+
+# This takes over an hour!
+RUN make -j ARCH=Linux-x86-64-nvhpc VERSION=ssmp cp2k
+"""
+
+
 # ======================================================================================
 def toolchain_cuda(gpu_ver: str) -> str:
     return rf"""
-FROM nvidia/cuda:11.3.1-devel-ubuntu20.04
+FROM nvidia/cuda:11.8.0-devel-ubuntu22.04
 
 # Setup CUDA environment.
 ENV CUDA_PATH /usr/local/cuda
 ENV LD_LIBRARY_PATH /usr/local/cuda/lib64
 
+# Disable JIT cache as there seems to be an issue with file locking on overlayfs.
+# See also https://github.com/cp2k/cp2k/pull/2337
+ENV CUDA_CACHE_DISABLE 1
+
 # Install Ubuntu packages.
 RUN apt-get update -qq && apt-get install -qq --no-install-recommends \
     gfortran                                                          \
@@ -461,7 +565,7 @@ RUN apt-get update -qq && apt-get install -qq --no-install-recommends \
 # ======================================================================================
 def toolchain_hip_cuda(gpu_ver: str) -> str:
     return rf"""
-FROM nvidia/cuda:11.3.1-devel-ubuntu20.04
+FROM nvidia/cuda:11.8.0-devel-ubuntu22.04
 
 # Setup CUDA environment.
 ENV CUDA_PATH /usr/local/cuda
@@ -471,6 +575,10 @@ ENV ROCM_VER 4.5.2
 ENV HIP_DIR /opt/HIP-rocm-4.5.2
 ENV HIPAMD_DIR /opt/hipamd-rocm-4.5.2
 
+# Disable JIT cache as there seems to be an issue with file locking on overlayfs.
+# See also https://github.com/cp2k/cp2k/pull/2337
+ENV CUDA_CACHE_DISABLE 1
+
 RUN export DEBIAN_FRONTEND=noninteractive DEBCONF_NONINTERACTIVE_SEEN=true \
     && apt-get update -qq && apt-get install -qq --no-install-recommends \
     ca-certificates \
@@ -564,7 +672,7 @@ RUN hipconfig
 # ======================================================================================
 def toolchain_hip_rocm(gpu_ver: str) -> str:
     return rf"""
-FROM rocm/dev-ubuntu-20.04:4.5.2-complete
+FROM rocm/dev-ubuntu-22.04:5.3.2-complete
 
 # Install some Ubuntu packages.
 RUN apt-get update -qq && apt-get install -qq --no-install-recommends \
@@ -587,7 +695,7 @@ RUN hipconfig
 
 
 # ======================================================================================
-def install_toolchain(base_image: str, **kwargs: Optional[str]) -> str:
+def install_toolchain(base_image: str, **kwargs: str) -> str:
     install_args = []
     for k, v in kwargs.items():
         k = k.replace("_", "-")
@@ -671,7 +779,7 @@ class OutputFile:
         output_path = Path(__file__).parent / self.filename
         if self.check:
             assert output_path.read_text(encoding="utf8") == self.content.getvalue()
-            print(f"File {output_path} is consisted with generator script.")
+            print(f"File {output_path} is consistent with generator script.")
         else:
             output_path.write_text(self.content.getvalue(), encoding="utf8")
             print(f"Wrote {output_path}")
