@@ -5,9 +5,9 @@
 # shellcheck disable=SC1091
 source /opt/cp2k-toolchain/install/setup
 
-cd /workspace/cp2k
+cd /opt/cp2k
 echo -n "Compiling cp2k... "
-if make -j VERSION=pdbg &> make.out; then
+if make -j VERSION=sdbg &> make.out; then
   echo "done."
 else
   echo -e "failed.\n\n"
@@ -17,12 +17,11 @@ else
   exit 0
 fi
 
-echo -e "\n========== Installing CP2K =========="
 cat > /usr/bin/cp2k_shell << EndOfMessage
 #!/bin/bash -e
 source /opt/cp2k-toolchain/install/setup
 export OMP_NUM_THREADS=1
-mpiexec -np 2 /workspace/cp2k/exe/local/cp2k_shell.pdbg "\$@"
+/opt/cp2k/exe/local/cp2k_shell.sdbg "\$@"
 EndOfMessage
 chmod +x /usr/bin/cp2k_shell
 
@@ -32,28 +31,43 @@ cat > /usr/bin/cp2k << EndOfMessage
 #!/bin/bash -e
 source /opt/cp2k-toolchain/install/setup
 export OMP_NUM_THREADS=1
-mpiexec -np 2 /workspace/cp2k/exe/local/cp2k.pdbg "\$@"
+/opt/cp2k/exe/local/cp2k.sdbg "\$@"
 EndOfMessage
 chmod +x /usr/bin/cp2k
 
 mkdir -p ~/.config/ase
-echo -e "[executables]\ncp2k = /usr/bin/cp2k_shell\n" > ~/.config/ase/ase.conf
+cat > ~/.config/ase/ase.conf << EndOfMessage
+[executables]
+cp2k = /usr/bin/cp2k_shell
+cp2k_main = /usr/bin/cp2k
+EndOfMessage
+
+echo -e "\n========== Installing Dependencies =========="
+apt-get update -qq
+apt-get install -qq --no-install-recommends \
+  python3 \
+  python3-dev \
+  python3-pip \
+  python3-wheel \
+  python3-setuptools \
+  build-essential
+rm -rf /var/lib/apt/lists/*
 
 echo -e "\n========== Installing ASE =========="
+git clone --quiet --depth=1 --single-branch -b master https://gitlab.com/ase/ase.git /opt/ase
 cd /opt/ase/
-git pull
 pip3 install ".[test]"
 
 echo -e "\n========== Running ASE Tests =========="
-cd /opt/ase/
 ASE_REVISION=$(git rev-parse --short HEAD)
+echo -
 
 if ase test -j 0 -c cp2k calculator/cp2k; then
-  echo "Summary: ASE commit ${ASE_REVISION} works fine."
-  echo "Status: OK"
+  echo -e "\nSummary: ASE commit ${ASE_REVISION} works fine."
+  echo -e "Status: OK\n"
 else
-  echo "Summary: Something is wrong with ASE commit ${ASE_REVISION}."
-  echo "Status: FAILED"
+  echo -e "\nSummary: Something is wrong with ASE commit ${ASE_REVISION}."
+  echo -e "Status: FAILED\n"
 fi
 
 #EOF
