@@ -6,8 +6,7 @@
 source /opt/cp2k-toolchain/install/setup
 
 echo -e "\n========== Compiling CP2K =========="
-cd /workspace/cp2k
-touch src/cp2k_info.F # ensure latest REVISION is picked up.
+cd /opt/cp2k
 echo -n "Compiling cp2k... "
 if make -j VERSION=psmp &> make.out; then
   echo "done."
@@ -19,15 +18,22 @@ else
   exit 0
 fi
 
+echo -e "\n========== Installing Dependencies =========="
+apt-get update -qq
+apt-get install -qq --no-install-recommends \
+  default-jre-headless \
+  libsaxonhe-java
+rm -rf /var/lib/apt/lists/*
+
 echo -e "\n========== Generating Manual =========="
 
 mkdir -p /workspace/artifacts/manual
 cd /workspace/artifacts/manual
 
-/workspace/cp2k/exe/local/cp2k.psmp --version
-/workspace/cp2k/exe/local/cp2k.psmp --xml
+/opt/cp2k/exe/local/cp2k.psmp --version
+/opt/cp2k/exe/local/cp2k.psmp --xml
 
-TOOLS=/workspace/cp2k/tools
+TOOLS=/opt/cp2k/tools
 cp ${TOOLS}/manual/favicon.png .
 cp ${TOOLS}/manual/toggle_folding.js .
 
@@ -41,13 +47,12 @@ set +e # disable error trapping for remainder of script
 )
 EXIT_CODE=$?
 
-echo ""
 if ((EXIT_CODE)); then
-  echo "Summary: Something is wrong."
-  echo "Status: FAILED"
+  echo -e "\nSummary: Something is wrong."
+  echo -e "Status: FAILED\n"
 else
-  echo "Summary: Manual generation works fine."
-  echo "Status: OK"
+  echo -e "\nSummary: Manual generation works fine."
+  echo -e "Status: OK\n"
 fi
 
 #EOF
