@@ -10,6 +10,15 @@ fi
 ARCH=$1
 VERSION=$2
 
+ulimit -c 0 # Disable core dumps as they can take a very long time to write.
+
+# Check available shared memory - needed for MPI inter-process communication.
+SHM_AVAIL=$(df --output=avail -m /dev/shm | tail -1)
+if ((SHM_AVAIL < 1024)); then
+  echo "ERROR: Not enough shared memory. If you're running docker use --shm-size=1g."
+  exit 1
+fi
+
 # shellcheck disable=SC1091
 source /opt/cp2k-toolchain/install/setup
 
@@ -19,6 +28,11 @@ if command -v ompi_info &> /dev/null; then
   export OMPI_MCA_plm_rsh_agent=/bin/false
 fi
 
+# Use keepalive mode for GPU tests.
+if [[ "${ARCH}" == *cuda* ]] || [[ "${ARCH}" == *hip* ]]; then
+  TESTOPTS="--keepalive ${TESTOPTS}"
+fi
+
 # Switch to stable DBCSR version if requested.
 if [ -n "${USE_STABLE_DBCSR}" ]; then
   echo "Switching to stable DBCSR version..."
@@ -32,7 +46,7 @@ fi
 
 # Compile cp2k.
 echo -en "\nCompiling cp2k... "
-cd /workspace/cp2k || exit 1
+cd /opt/cp2k || exit 1
 if make -j ARCH="${ARCH}" VERSION="${VERSION}" &> make.out; then
   echo "done."
 else
@@ -43,19 +57,15 @@ else
   exit 0
 fi
 
-# Check benchmark files for input errors.
-if [[ "${ARCH}" == "local" ]]; then
-  echo -en "\nChecking benchmarks... "
-  if ! ./tools/regtesting/check_inputs.py "./exe/${ARCH}/cp2k.${VERSION}" "./benchmarks/"; then
-    echo -e "\nSummary: Some benchmark inputs could not be parsed."
-    echo -e "Status: FAILED\n"
-    exit 0
-  fi
-fi
-
 # Improve code coverage on COSMA.
 export COSMA_DIM_THRESHOLD=0
 
+# Extend stack size only for Intel compiler - otherwise it breaks tests on i386.
+if "./exe/${ARCH}/cp2k.${VERSION}" --version | grep -q "compiler: Intel"; then
+  ulimit -s unlimited
+  export OMP_STACKSIZE=64m
+fi
+
 # Run regtests.
 echo -e "\n========== Running Regtests =========="
 make ARCH="${ARCH}" VERSION="${VERSION}" TESTOPTS="${TESTOPTS}" test
