@@ -79,7 +79,7 @@ def parse_data_set(input_path: Path) -> List[Approximation]:
                 if err == 0.0 or k >= 58:
                     continue
                 # slight change of notation e.g. 6E09 --> 6_9
-                Rc_filename = re.sub("E0?(?=\d)", "_", Rc_str.strip())
+                Rc_filename = re.sub(r"E0?(?=\d)", "_", Rc_str.strip())
                 filename = input_path / f"1_xk{k:02d}.{Rc_filename}"
                 assert filename.exists()
 
@@ -204,7 +204,7 @@ def generate_fortran_code(approx: List[Approximation]) -> str:
     output += [
         "!--------------------------------------------------------------------------------------------------!",
         "!   CP2K: A general program to perform molecular dynamics simulations                              !",
-        "!   Copyright 2000-2021 CP2K developers group <https://cp2k.org>                                   !",
+        "!   Copyright 2000-2022 CP2K developers group <https://cp2k.org>                                   !",
         "!                                                                                                  !",
         "!   SPDX-License-Identifier: GPL-2.0-or-later                                                      !",
         "!--------------------------------------------------------------------------------------------------!",
