@@ -33,25 +33,31 @@ TR=$(command -v tr)
 if [ "" != "${HOME}" ]; then
   CONFIGFILE=${HOME}/.xconfigure-cp2k-plan
 else
-  HERE=$(cd $(dirname $0); pwd -P)
+  HERE=$(
+    cd "$(dirname "$0")" || exit
+    pwd -P
+  )
   CONFIGFILE=${HERE}/.xconfigure-cp2k-plan
 fi
 
 function isqrt {
-  s=1073741824; x=$1; y=0
+  s=1073741824
+  x=$1
+  y=0
   while [ "0" != "$((0 < s))" ]; do
-    b=$((y | s)); y=$((y >> 1))
+    b=$((y | s))
+    y=$((y >> 1))
     if [ "0" != "$((b <= x))" ]; then
-      x=$((x - b)); y=$((y | s))
+      x=$((x - b))
+      y=$((y | s))
     fi
     s=$((s >> 2))
   done
   echo "${y}"
 }
 
-if [ "" != "${GREP}" ] && [ "" != "${SORT}" ] && [ "" != "${HEAD}" ] && \
-   [ "" != "${SEQ}" ] && [ "" != "${CUT}" ] && [ "" != "${TR}" ];
-then
+if [ "" != "${GREP}" ] && [ "" != "${SORT}" ] && [ "" != "${HEAD}" ] &&
+  [ "" != "${SEQ}" ] && [ "" != "${CUT}" ] && [ "" != "${TR}" ]; then
   HELP=0
   if [ "--help" = "$1" ] || [ "-help" = "$1" ] || [ "-h" = "$1" ]; then
     HELP=1
@@ -60,10 +66,9 @@ then
     TOTALNUMNODES=$1
     shift
   fi
-  if [ -e /proc/cpuinfo ] && [ "" != "$(command -v wc)" ];
-  then
+  if [ -e /proc/cpuinfo ] && [ "" != "$(command -v wc)" ]; then
     NS=$(${GREP} "physical id" /proc/cpuinfo | ${SORT} -u | wc -l | ${TR} -d " ")
-    NC=$((NS*$(${GREP} -m1 "cpu cores" /proc/cpuinfo | ${TR} -d " " | ${CUT} -d: -f2)))
+    NC=$((NS * $(${GREP} -m1 "cpu cores" /proc/cpuinfo | ${TR} -d " " | ${CUT} -d: -f2)))
     NT=$(${GREP} "core id" /proc/cpuinfo | wc -l | ${TR} -d " ")
   elif [ "Darwin" = "$(uname)" ] && [ "" != "$(command -v sysctl)" ]; then
     NS=$(sysctl hw.packages | ${CUT} -d: -f2 | ${TR} -d " ")
@@ -71,12 +76,12 @@ then
     NT=$(sysctl hw.logicalcpu | ${CUT} -d: -f2 | ${TR} -d " ")
   fi
   if [ "" != "${NC}" ] && [ "" != "${NT}" ]; then
-    HT=$((NT/NC))
+    HT=$((NT / NC))
   fi
   OUTPUT=0
   if [ "" = "$1" ]; then
-    if [ -e ${CONFIGFILE} ]; then  # remind configuration
-      NCORESPERNODE=$(${CUT} -d" " -f1 ${CONFIGFILE})
+    if [ -e "${CONFIGFILE}" ]; then # remind configuration
+      NCORESPERNODE=$(${CUT} -d" " -f1 "${CONFIGFILE}")
     elif [ "" != "${NC}" ]; then
       NCORESPERNODE=${NC}
     fi
@@ -86,8 +91,8 @@ then
     shift
   fi
   if [ "" = "$1" ]; then
-    if [ -e ${CONFIGFILE} ]; then  # remind configuration
-      NTHREADSPERCORE=$(${CUT} -d" " -f2 ${CONFIGFILE})
+    if [ -e "${CONFIGFILE}" ]; then # remind configuration
+      NTHREADSPERCORE=$(${CUT} -d" " -f2 "${CONFIGFILE}")
     elif [ "" != "${HT}" ]; then
       NTHREADSPERCORE=${HT}
     fi
@@ -97,8 +102,8 @@ then
     shift
   fi
   if [ "" = "$1" ]; then
-    if [ -e ${CONFIGFILE} ]; then  # remind configuration
-      NPROCSPERNODE=$(${CUT} -d" " -f3 ${CONFIGFILE})
+    if [ -e "${CONFIGFILE}" ]; then # remind configuration
+      NPROCSPERNODE=$(${CUT} -d" " -f3 "${CONFIGFILE}")
     elif [ "" != "${NS}" ]; then
       NPROCSPERNODE=${NS}
     fi
@@ -120,31 +125,30 @@ then
     exit 0
   fi
   # min. number of ranks per node
-  MIN_NRANKS=$((PENALTY_MIN*NPROCSPERNODE))
+  MIN_NRANKS=$((PENALTY_MIN * NPROCSPERNODE))
   # remember system configuration
   if [ "0" != "${OUTPUT}" ]; then
-    echo "${NCORESPERNODE} ${NTHREADSPERCORE} ${NPROCSPERNODE}" > ${CONFIGFILE} 2> /dev/null
+    echo "${NCORESPERNODE} ${NTHREADSPERCORE} ${NPROCSPERNODE}" > "${CONFIGFILE}" 2> /dev/null
   fi
-  NCORESTOTAL=$((TOTALNUMNODES*NCORESPERNODE))
-  NCORESOCKET=$((NCORESPERNODE/NPROCSPERNODE))
+  NCORESTOTAL=$((TOTALNUMNODES * NCORESPERNODE))
+  NCORESOCKET=$((NCORESPERNODE / NPROCSPERNODE))
   echo "================================================================================"
   echo "${NCORESTOTAL} cores: ${TOTALNUMNODES} node(s) with ${NPROCSPERNODE}x${NCORESOCKET} core(s) per node and ${NTHREADSPERCORE} thread(s) per core"
   echo "================================================================================"
-  NRANKSMIN=$((TOTALNUMNODES*NPROCSPERNODE))
+  NRANKSMIN=$((TOTALNUMNODES * NPROCSPERNODE))
   NSQRT_MIN=$(isqrt ${NRANKSMIN})
   NSQRT_MAX=$(isqrt ${NCORESTOTAL})
-  for NSQRT in $(${SEQ} ${NSQRT_MIN} ${NSQRT_MAX}); do
-    NSQR=$((NSQRT*NSQRT))
-    NRANKSPERNODE=$((NSQR/TOTALNUMNODES))
-    if [ "${NSQR}" = "$((TOTALNUMNODES*NRANKSPERNODE))" ]; then
-      REST=$((NCORESPERNODE%NRANKSPERNODE))
-      ODD=$(((NRANKSPERNODE%NPROCSPERNODE)!=0))
+  for NSQRT in $(${SEQ} "${NSQRT_MIN}" "${NSQRT_MAX}"); do
+    NSQR=$((NSQRT * NSQRT))
+    NRANKSPERNODE=$((NSQR / TOTALNUMNODES))
+    if [ "${NSQR}" = "$((TOTALNUMNODES * NRANKSPERNODE))" ]; then
+      REST=$((NCORESPERNODE % NRANKSPERNODE))
+      ODD=$(((NRANKSPERNODE % NPROCSPERNODE) != 0))
       # criterion to add penalty in case of unbalanced load
-      if [ "0" != "$((PENALTY_ODD*MIN_NRANKS*REST <= NCORESPERNODE))" ] || [ "0" = "${ODD}" ]; then
-        if [ "0" != "$((MIN_NRANKS*REST <= NCORESPERNODE))" ] && \
-           [ "0" != "$((MIN_NRANKS <= NRANKSPERNODE))" ];
-        then
-          PENALTY=$(((100*REST+ODD*PENALTY_ODD+NCORESPERNODE-1)/NCORESPERNODE))
+      if [ "0" != "$((PENALTY_ODD * MIN_NRANKS * REST <= NCORESPERNODE))" ] || [ "0" = "${ODD}" ]; then
+        if [ "0" != "$((MIN_NRANKS * REST <= NCORESPERNODE))" ] &&
+          [ "0" != "$((MIN_NRANKS <= NRANKSPERNODE))" ]; then
+          PENALTY=$(((100 * REST + ODD * PENALTY_ODD + NCORESPERNODE - 1) / NCORESPERNODE))
           RESULTS+="${NRANKSPERNODE};${PENALTY};${NSQRT}\n"
         fi
       fi
@@ -152,28 +156,27 @@ then
   done
   RESULTS=$(echo -e "${RESULTS}" | ${GREP} -v "^$" | ${SORT} -t";" -u -k2n -k1nr)
   NRANKSPERNODE_TOP=$(echo "${RESULTS}" | ${CUT} -d";" -f1 | ${HEAD} -n1)
-  NTHREADSPERNODE=$((NCORESPERNODE*NTHREADSPERCORE))
-  NSQR_MAX=$((NSQRT_MAX*NSQRT_MAX))
-  PENALTY_NCORES=$((NCORESTOTAL-NSQR_MAX))
-  PENALTY_TOP=$(((100*PENALTY_NCORES+NCORESTOTAL-1)/NCORESTOTAL))
+  NTHREADSPERNODE=$((NCORESPERNODE * NTHREADSPERCORE))
+  NSQR_MAX=$((NSQRT_MAX * NSQRT_MAX))
+  PENALTY_NCORES=$((NCORESTOTAL - NSQR_MAX))
+  PENALTY_TOP=$(((100 * PENALTY_NCORES + NCORESTOTAL - 1) / NCORESTOTAL))
   NRANKSPERNODE=${NCORESPERNODE}
   OUTPUT_POT=0
   while [ "0" != "$((NRANKSPERNODE_TOP < NRANKSPERNODE))" ]; do
     # criterion to add penalty in case of unbalanced load
-    ODD=$(((NRANKSPERNODE%NPROCSPERNODE)!=0))
-    if [ "0" != "$((PENALTY_ODD*MIN_NRANKS*PENALTY_NCORES <= NCORESTOTAL))" ] || [ "0" = "${ODD}" ]; then
-      NTHREADSPERRANK=$((NTHREADSPERNODE/NRANKSPERNODE))
-      if [ "0" != "$((MIN_NRANKS*PENALTY_NCORES <= NCORESTOTAL))" ] && \
-         [ "0" != "$((MIN_NRANKS <= NRANKSPERNODE))" ];
-      then
-        NRANKS_CUR=$((TOTALNUMNODES*NRANKSPERNODE))
+    ODD=$(((NRANKSPERNODE % NPROCSPERNODE) != 0))
+    if [ "0" != "$((PENALTY_ODD * MIN_NRANKS * PENALTY_NCORES <= NCORESTOTAL))" ] || [ "0" = "${ODD}" ]; then
+      NTHREADSPERRANK=$((NTHREADSPERNODE / NRANKSPERNODE))
+      if [ "0" != "$((MIN_NRANKS * PENALTY_NCORES <= NCORESTOTAL))" ] &&
+        [ "0" != "$((MIN_NRANKS <= NRANKSPERNODE))" ]; then
+        NRANKS_CUR=$((TOTALNUMNODES * NRANKSPERNODE))
         SQRTNRANKS=$(isqrt $((NRANKS_CUR)))
-        NRANKS_SQR=$((SQRTNRANKS*SQRTNRANKS))
-        NRANKS_COM=$((NRANKSPERNODE<NRANKS_SQR?(NRANKS_SQR/NRANKSPERNODE*NRANKSPERNODE):NRANKS_SQR))
-        PENALTY=$((100*(NRANKS_CUR+ODD*PENALTY_ODD-NRANKS_COM)/NRANKS_COM))
+        NRANKS_SQR=$((SQRTNRANKS * SQRTNRANKS))
+        NRANKS_COM=$((NRANKSPERNODE < NRANKS_SQR ? (NRANKS_SQR / NRANKSPERNODE * NRANKSPERNODE) : NRANKS_SQR))
+        PENALTY=$((100 * (NRANKS_CUR + ODD * PENALTY_ODD - NRANKS_COM) / NRANKS_COM))
         echo "[${NRANKSPERNODE}x${NTHREADSPERRANK}]: ${NRANKSPERNODE} ranks per node with ${NTHREADSPERRANK} thread(s) per rank (${PENALTY}% penalty)"
         if [ "0" != "$((PENALTY_TOP < PENALTY))" ]; then PENALTY_TOP=${PENALTY}; fi
-        OUTPUT_POT=$((OUTPUT_POT+1))
+        OUTPUT_POT=$((OUTPUT_POT + 1))
       fi
     fi
     NRANKSPERNODE=$((NRANKSPERNODE >> 1))
@@ -186,59 +189,56 @@ then
   RESULTS=$(echo -e "${RESULTS}" | ${TR} " " "\n" | ${SORT} -t";" -k1nr -k2n)
   for RESULT in ${RESULTS}; do
     NRANKSPERNODE=$(echo "${RESULT}" | ${CUT} -d";" -f1)
-    NTHREADSPERRANK=$((NTHREADSPERNODE/NRANKSPERNODE))
+    NTHREADSPERRANK=$((NTHREADSPERNODE / NRANKSPERNODE))
     PENALTY=$(echo "${RESULT}" | ${CUT} -d";" -f2)
-    if [ "0" != "$((OUTPUT_SQR < OUTPUT_POT))" ] || \
-       [ "0" != "$((PENALTY <= PENALTY_TOP))" ];
-    then
+    if [ "$((OUTPUT_SQR < OUTPUT_POT))" != "0" ] ||
+      [ "$((PENALTY <= PENALTY_TOP))" != "0" ]; then
       NSQRT=$(echo "${RESULT}" | ${CUT} -d";" -f3)
       echo "[${NRANKSPERNODE}x${NTHREADSPERRANK}]: ${NRANKSPERNODE} ranks per node with ${NTHREADSPERRANK} thread(s) per rank (${PENALTY}% penalty) -> ${NSQRT}x${NSQRT}"
-      OUTPUT_SQR=$((OUTPUT_SQR+1))
+      OUTPUT_SQR=$((OUTPUT_SQR + 1))
     fi
   done
   if [ "0" != "${OUTPUT_SQR}" ]; then
     echo "--------------------------------------------------------------------------------"
   fi
-  for NRANKSPERNODE in $(${SEQ} ${MIN_NRANKS} ${NCORESPERNODE}); do
-    REST=$((NCORESPERNODE%NRANKSPERNODE))
-    PENALTY=$(((100*REST+NCORESPERNODE-1)/NCORESPERNODE))
+  for NRANKSPERNODE in $(${SEQ} "${MIN_NRANKS}" "${NCORESPERNODE}"); do
+    REST=$((NCORESPERNODE % NRANKSPERNODE))
+    PENALTY=$(((100 * REST + NCORESPERNODE - 1) / NCORESPERNODE))
     # criterion to add penalty in case of unbalanced load
-    ODD=$(((NRANKSPERNODE%NPROCSPERNODE)!=0))
-    if [ "0" != "$((PENALTY_ODD*MIN_NRANKS*REST <= NCORESPERNODE))" ] || [ "0" = "${ODD}" ];
-    then
-      if [ "0" != "$((MIN_NRANKS*REST <= NCORESPERNODE))" ] && \
-         [ "0" != "$((MIN_NRANKS <= NRANKSPERNODE))" ];
-      then
-        SQRT=$(isqrt $((TOTALNUMNODES*NRANKSPERNODE)))
-        NUMCORES=$((SQRT*SQRT))
-        NUMNODES=$((NUMCORES/NRANKSPERNODE))
-        if [ "0" = "$((NUMCORES-NUMNODES*NRANKSPERNODE))" ]; then
+    ODD=$(((NRANKSPERNODE % NPROCSPERNODE) != 0))
+    if [ "0" != "$((PENALTY_ODD * MIN_NRANKS * REST <= NCORESPERNODE))" ] || [ "0" = "${ODD}" ]; then
+      if [ "0" != "$((MIN_NRANKS * REST <= NCORESPERNODE))" ] &&
+        [ "0" != "$((MIN_NRANKS <= NRANKSPERNODE))" ]; then
+        SQRT=$(isqrt $((TOTALNUMNODES * NRANKSPERNODE)))
+        NUMCORES=$((SQRT * SQRT))
+        NUMNODES=$((NUMCORES / NRANKSPERNODE))
+        if [ "0" = "$((NUMCORES - NUMNODES * NRANKSPERNODE))" ]; then
           SUGGEST_LO+="${NUMNODES}\n"
         fi
-        NUMCORES=$(((SQRT-1)*(SQRT-1)))
-        NUMNODES=$((NUMCORES/NRANKSPERNODE))
-        if [ "0" = "$((NUMCORES-NUMNODES*NRANKSPERNODE))" ]; then
+        NUMCORES=$(((SQRT - 1) * (SQRT - 1)))
+        NUMNODES=$((NUMCORES / NRANKSPERNODE))
+        if [ "0" = "$((NUMCORES - NUMNODES * NRANKSPERNODE))" ]; then
           SUGGEST_LO+="${NUMNODES}\n"
         fi
-        NUMCORES=$(((SQRT+1)*(SQRT+1)))
-        NUMNODES=$((NUMCORES/NRANKSPERNODE))
-        if [ "0" = "$((NUMCORES-NUMNODES*NRANKSPERNODE))" ]; then
+        NUMCORES=$(((SQRT + 1) * (SQRT + 1)))
+        NUMNODES=$((NUMCORES / NRANKSPERNODE))
+        if [ "0" = "$((NUMCORES - NUMNODES * NRANKSPERNODE))" ]; then
           SUGGEST_HI+="${NUMNODES}\n"
         fi
-        NUMCORES=$(((SQRT+2)*(SQRT+2)))
-        NUMNODES=$((NUMCORES/NRANKSPERNODE))
-        if [ "0" = "$((NUMCORES-NUMNODES*NRANKSPERNODE))" ] && [ "" = "${SUGGEST_HI}" ]; then
+        NUMCORES=$(((SQRT + 2) * (SQRT + 2)))
+        NUMNODES=$((NUMCORES / NRANKSPERNODE))
+        if [ "0" = "$((NUMCORES - NUMNODES * NRANKSPERNODE))" ] && [ "" = "${SUGGEST_HI}" ]; then
           SUGGEST_HI+="${NUMNODES}\n"
         fi
       fi
     fi
   done
-  SUGGEST_LO=$(echo -e "${SUGGEST_LO}" \
-    | ${GREP} -vw "${TOTALNUMNODES}" \
-    | ${SORT} -nr | ${GREP} -v "^$" -m1)
-  SUGGEST_HI=$(echo -e "${SUGGEST_HI}" \
-    | ${GREP} -vw "${TOTALNUMNODES}" \
-    | ${SORT} -n | ${GREP} -v "^$" -m1)
+  SUGGEST_LO=$(echo -e "${SUGGEST_LO}" |
+    ${GREP} -vw "${TOTALNUMNODES}" |
+    ${SORT} -nr | ${GREP} -v "^$" -m1)
+  SUGGEST_HI=$(echo -e "${SUGGEST_HI}" |
+    ${GREP} -vw "${TOTALNUMNODES}" |
+    ${SORT} -n | ${GREP} -v "^$" -m1)
   if [ "0" != "${SUGGEST_LO}" ]; then
     echo "Try also the following node counts: ${SUGGEST_LO} ${SUGGEST_HI}"
   else
@@ -248,4 +248,3 @@ else
   echo "Error: missing prerequisites!"
   exit 1
 fi
-
