@@ -41,9 +41,10 @@ def shellcheck():
 
 
 # ======================================================================================
-@app.route("/markdownlint", methods=["POST"])
-def markdownlint():
-    return run_tool(["markdownlint"])
+@app.route("/mdformat", methods=["POST"])
+@app.route("/markdownlint", methods=["POST"])  # for backwards compatibility
+def mdformat():
+    return run_tool(["mdformat"])
 
 
 # ======================================================================================
