@@ -33,17 +33,8 @@ import difflib
 import subprocess
 import inspect
 
-# allow unicode for stdin / stdout, it's a mess
-try:
-    # python 3
-    sys.stderr = io.TextIOWrapper(
-        sys.stderr.detach(), encoding='UTF-8', line_buffering=True)
-
-except AttributeError: # pragma: no cover
-    # python 2
-    import codecs
-    utf8_writer = codecs.getwriter('UTF-8')
-    sys.stderr = utf8_writer(sys.stderr)
+sys.stderr = io.TextIOWrapper(
+    sys.stderr.detach(), encoding='UTF-8', line_buffering=True)
 
 import fprettify
 from fprettify.fparse_utils import FprettifyParseException, FprettifyInternalException
@@ -76,9 +67,7 @@ def eprint(*args, **kwargs):
     Print to stderr - to print output compatible with default unittest output.
     """
 
-    print(*args, file=sys.stderr, **kwargs)
-    sys.stderr.flush()  # python 2 print does not have flush argument
-
+    print(*args, file=sys.stderr, flush=True, **kwargs)
 
 class FPrettifyTestCase(unittest.TestCase):
     """
@@ -186,15 +175,27 @@ class FPrettifyTestCase(unittest.TestCase):
         self.assert_fprettify_result([], instring, outstring_exp_default)
         self.assert_fprettify_result(['--strict-indent'], instring, outstring_exp_strict)
 
+    def test_reset_indent(self):
+        """test of reset indentation at file start"""
+        instring = ("integer :: i,j\ndo i=1,2\ndo j=1,3\n"
+                    "print*,i,j,i*j\nend do\nend do",
+                    "   module a\ninteger :: 1\n")
+        outstring = ("integer :: i, j\ndo i = 1, 2\ndo j = 1, 3\n"
+                     "   print *, i, j, i*j\nend do\nend do",
+                     "module a\n   integer :: 1")
+
+        for ind, out in zip(instring, outstring):
+            self.assert_fprettify_result([],ind, out)
+
     def test_disable(self):
         """test disabling indentation and/or whitespace formatting"""
-        instring = ("if(&\nl==111)&\n then\n   do m   =1,  2\n A=&\nB+C\n    enddo;   endif")
+        instring = ("if(&\nl==111)&\n then\n   do m   =1,  2\n A=&\nB+C\n    end  do;   endif")
         outstring_exp_default = ("if ( &\n   l == 111) &\n   then\n   do m = 1, 2\n"
-                                 "      A = &\n         B + C\n   enddo; endif")
+                                 "      A = &\n         B + C\n   end do; end if")
         outstring_exp_nowhitespace = ("if(&\n   l==111)&\n   then\n   do m   =1,  2\n"
-                                      "      A=&\n         B+C\n   enddo; endif")
+                                      "      A=&\n         B+C\n   end  do; endif")
         outstring_exp_noindent = ("if ( &\nl == 111) &\n then\n   do m = 1, 2\n"
-                                  " A = &\nB + C\n    enddo;   endif")
+                                  " A = &\nB + C\n    end do;   end if")
 
         self.assert_fprettify_result([], instring, outstring_exp_default)
         self.assert_fprettify_result(['--disable-whitespace'], instring, outstring_exp_nowhitespace)
@@ -246,8 +247,8 @@ class FPrettifyTestCase(unittest.TestCase):
         p1 = subprocess.Popen(
             args, stdout=subprocess.PIPE, stdin=subprocess.PIPE)
         outstring = p1.communicate(instring.encode(
-            'UTF-8'))[0].decode('UTF-8').strip()
-        self.assertEqual(outstring_exp, outstring)
+            'UTF-8'))[0].decode('UTF-8').rstrip()
+        self.assertEqual(outstring_exp.rstrip(), outstring)
 
     def test_io(self):
         """simple test for io (file inplace, stdin & stdout)"""
@@ -335,6 +336,17 @@ class FPrettifyTestCase(unittest.TestCase):
         for i in range(0, len(instring)):
             self.assert_fprettify_result([], instring[i], outstring[i])
 
+    def test_associate(self):
+        """test correct formatting of associate construct"""
+        instring = ("associate(a=>b , c  =>d ,e=> f  )\n"
+                    "e=a+c\n"
+                    "end associate")
+        outstring = ("associate (a => b, c => d, e => f)\n"
+                    "   e = a + c\n"
+                    "end associate")
+
+        self.assert_fprettify_result([], instring, outstring)
+
     def test_line_length(self):
         """test line length option"""
         instring = ["REAL(KIND=4) :: r,f  !  some reals",
@@ -359,7 +371,7 @@ class FPrettifyTestCase(unittest.TestCase):
         self.assert_fprettify_result(['-S', '-l 0'], instring_, outstring_[1])
 
     def test_relation_replacement(self):
-        """test relacement of relational statements"""
+        """test replacement of relational statements"""
         instring = ["if ( min < max .and. min .lt. thres)",
                     "if (min > max .and. min .gt. thres )",
                     "if (   min == max .and. min .eq. thres  )",
@@ -376,24 +388,72 @@ class FPrettifyTestCase(unittest.TestCase):
                      "if (min .ne. max .and. min .ne. thres)",
                      "if (min .ge. max .and. min .ge. thres)",
                      "if (min .le. max .and. min .le. thres)",
-                    "'==== heading",
-                    "if (vtk%my_rank .eq. 0) write (vtk%filehandle_par, '(\"<DataArray",
-                    "'(\"</Collection>\",",
-                    "if (abc(1) .lt. -bca .or. &\n    qwe .gt. ewq) then"]
+                     "'==== heading",
+                     "if (vtk%my_rank .eq. 0) write (vtk%filehandle_par, '(\"<DataArray",
+                     "'(\"</Collection>\",",
+                     "if (abc(1) .lt. -bca .or. &\n    qwe .gt. ewq) then"]
         c_outstring = ["if (min < max .and. min < thres)",
                      "if (min > max .and. min > thres)",
                      "if (min == max .and. min == thres)",
                      "if (min /= max .and. min /= thres)",
                      "if (min >= max .and. min >= thres)",
                      "if (min <= max .and. min <= thres)",
-                    "'==== heading",
-                    "if (vtk%my_rank == 0) write (vtk%filehandle_par, '(\"<DataArray",
-                     "'(\"</Collection>\",",
-                    "if (abc(1) < -bca .or. &\n    qwe > ewq) then"]
+                     "'==== heading",
+                     "if (vtk%my_rank == 0) write (vtk%filehandle_par, '(\"<DataArray",
+                      "'(\"</Collection>\",",
+                     "if (abc(1) < -bca .or. &\n    qwe > ewq) then"]
         for i in range(0, len(instring)):
             self.assert_fprettify_result(['--enable-replacements', '--c-relations'], instring[i], c_outstring[i])
             self.assert_fprettify_result(['--enable-replacements'], instring[i], f_outstring[i])
 
+    def test_swap_case(self):
+        """test replacement of keyword character case"""
+        instring = (
+            "MODULE exAmple",
+            "INTEGER,   PARAMETER :: SELECTED_REAL_KIND = 1*2",
+            "INTEGER,   PARAMETER :: dp1 = SELECTED_REAL_KIND ( 15 , 307)",
+            'CHARACTER(LEN=*), PARAMETER :: a = "INTEGER,   PARAMETER" // "b"',
+            "CHARACTER(LEN=*), PARAMETER :: a = 'INTEGER,   PARAMETER' // 'b'",
+            "INTEGER(kind=int64), PARAMETER :: l64 = 2_int64",
+            "REAL(kind=real64), PARAMETER :: r64a = 2._real64",
+            "REAL(kind=real64), PARAMETER :: r64b = 2.0_real64",
+            "REAL(kind=real64), PARAMETER :: r64c = .0_real64",
+            "REAL(kind=real64), PARAMETER :: r64a = 2.e3_real64",
+            "REAL(kind=real64), PARAMETER :: r64b = 2.0e3_real64",
+            "REAL(kind=real64), PARAMETER :: r64c = .0e3_real64",
+            "REAL, PARAMETER :: r32 = 2.e3",
+            "REAL, PARAMETER :: r32 = 2.0d3",
+            "REAL, PARAMETER :: r32 = .2e3",
+            "USE ISO_FORTRAN_ENV, ONLY: int64",
+            "INTEGER, INTENT(IN) :: r, i, j, k",
+            "IF (l.EQ.2) l=MAX  (l64, 2_int64)",
+            "PURE SUBROUTINE mypure()"
+            )
+        outstring = (
+            "module exAmple",
+            "integer, parameter :: SELECTED_REAL_KIND = 1*2",
+            "integer, parameter :: dp1 = selected_real_kind(15, 307)",
+            'character(LEN=*), parameter :: a = "INTEGER,   PARAMETER"//"b"',
+            "character(LEN=*), parameter :: a = 'INTEGER,   PARAMETER'//'b'",
+            "integer(kind=INT64), parameter :: l64 = 2_INT64",
+            "real(kind=REAL64), parameter :: r64a = 2._REAL64",
+            "real(kind=REAL64), parameter :: r64b = 2.0_REAL64",
+            "real(kind=REAL64), parameter :: r64c = .0_REAL64",
+            "real(kind=REAL64), parameter :: r64a = 2.E3_REAL64",
+            "real(kind=REAL64), parameter :: r64b = 2.0E3_REAL64",
+            "real(kind=REAL64), parameter :: r64c = .0E3_REAL64",
+            "real, parameter :: r32 = 2.E3",
+            "real, parameter :: r32 = 2.0D3",
+            "real, parameter :: r32 = .2E3",
+            "use iso_fortran_env, only: INT64",
+            "integer, intent(IN) :: r, i, j, k",
+            "if (l .eq. 2) l = max(l64, 2_INT64)",
+            "pure subroutine mypure()"
+            )
+        for i in range(len(instring)):
+            self.assert_fprettify_result(['--case', '1', '1', '1', '2'],
+                                         instring[i], outstring[i])
+
     def test_do(self):
         """test correct parsing of do statement"""
         instring = "do = 1\nb = 2"
@@ -423,6 +483,362 @@ class FPrettifyTestCase(unittest.TestCase):
 
         self.assert_fprettify_result([], instring, outstring)
 
+    def test_ford(self):
+        """test formatting of ford comments"""
+        instring =  ("   a = b\n"
+                     "     !!  ford docu\n"
+                     "b=c\n"
+                     "  !! ford docu\n"
+                     "subroutine test(a,b,&\n"
+                     "  !! ford docu\n"
+                     "  c, d, e)"
+                     )
+        outstring = ("   a = b\n"
+                     "     !!  ford docu\n"
+                     "   b = c\n"
+                     "  !! ford docu\n"
+                     "   subroutine test(a, b, &\n"
+                     "  !! ford docu\n"
+                     "                   c, d, e)"
+                     )
+
+        self.assert_fprettify_result([], instring, outstring)
+
+    def test_plusminus(self):
+        """test corner cases of +/-"""
+        instring = "val_1d-1-1.0e-9-2.0d-08+.2e-1-val_2d-3.e-12+4"
+        outstring = "val_1d - 1 - 1.0e-9 - 2.0d-08 + .2e-1 - val_2d - 3.e-12 + 4"
+        self.assert_fprettify_result([], instring, outstring)
+
+    def test_fypp(self):
+        """test formatting of fypp statements"""
+
+        instring = []
+        outstring = []
+
+        instring += [
+"""
+#:if DEBUG>  0
+print *, "hola"
+if( .not. (${cond}$) ) then
+#:if  ASSERT(cond)
+print *, "Assert failed!"
+#:endif
+error stop
+end if
+#:endif
+"""
+]
+
+        outstring += [
+"""
+#:if DEBUG>  0
+   print *, "hola"
+   if (.not. (${cond}$)) then
+      #:if  ASSERT(cond)
+         print *, "Assert failed!"
+      #:endif
+      error stop
+   end if
+#:endif
+"""
+]
+
+        instring += [
+"""
+if  (.not. (${cond}$)) then
+   #:for element in list
+   print *, "Element is in list!"
+ #:endfor
+   error stop
+end if
+"""
+]
+
+        outstring += [
+"""
+if (.not. (${cond}$)) then
+   #:for element in list
+      print *, "Element is in list!"
+   #:endfor
+   error stop
+end if
+"""
+]
+
+        instring += [
+"""
+#:if aa > 1
+print  *, "Number is more than 1"
+if (condition) then
+     #:def something
+   print *, "Added Definition!"
+   #:enddef
+end if
+#:endif
+"""
+]
+
+        outstring += [
+"""
+#:if aa > 1
+   print *, "Number is more than 1"
+   if (condition) then
+      #:def something
+         print *, "Added Definition!"
+      #:enddef
+   end if
+#:endif
+"""
+]
+
+        instring += [
+"""
+#:def DEBUG_CODE( code)
+  #:if DEBUG > 0
+    $:code
+  #:endif
+#:enddef DEBUG_CODE
+"""
+]
+
+        outstring += [
+"""
+#:def DEBUG_CODE( code)
+   #:if DEBUG > 0
+      $:code
+   #:endif
+#:enddef DEBUG_CODE
+"""
+]
+
+
+        instring += [
+"""
+#:block DEBUG_CODE
+  if (a <b) then
+    print *, "DEBUG: a is less than b"
+  end if
+#:endblock  DEBUG_CODE
+"""
+]
+
+        outstring += [
+"""
+#:block DEBUG_CODE
+   if (a < b) then
+      print *, "DEBUG: a is less than b"
+   end if
+#:endblock  DEBUG_CODE
+"""
+]
+
+        instring += [
+"""
+#:call DEBUG_CODE
+  if (a < b) then
+    print *, "DEBUG: a is less than b"
+  end if
+#:endcall DEBUG_CODE
+"""
+]
+
+        outstring += [
+"""
+#:call DEBUG_CODE
+   if (a < b) then
+      print *, "DEBUG: a is less than b"
+   end if
+#:endcall DEBUG_CODE
+"""
+]
+
+        instring += [
+"""
+#:if DEBUG > 0
+print *, "hola"
+if (.not. (${cond}$)) then
+   #:mute
+   print *, "Muted"
+   #:endmute
+   error stop
+end if
+#:endif
+"""
+]
+
+        outstring += [
+"""
+#:if DEBUG > 0
+   print *, "hola"
+   if (.not. (${cond}$)) then
+      #:mute
+         print *, "Muted"
+      #:endmute
+      error stop
+   end if
+#:endif
+"""
+]
+
+        instring += [
+"""
+program try
+#:def mydef
+a = &
+#:if dothat
+b + &
+#:else
+c + &
+#:endif
+d
+#:enddef
+end program
+"""
+]
+
+        outstring += [
+"""
+program try
+   #:def mydef
+      a = &
+#:if dothat
+         b + &
+#:else
+         c + &
+#:endif
+         d
+   #:enddef
+end program
+"""
+]
+
+        instring += [
+"""
+#:if worktype
+      ${worktype}$, &
+#:else
+      ${type}$, &
+#:endif
+         DIMENSION(${arr_exp}$), &
+         POINTER :: work
+"""
+]
+
+        outstring += [
+"""
+#:if worktype
+${worktype}$, &
+#:else
+   ${type}$, &
+#:endif
+   DIMENSION(${arr_exp}$), &
+   POINTER :: work
+"""
+]
+
+
+
+        for instr, outstr in zip(instring, outstring):
+            self.assert_fprettify_result([], instr, outstr)
+
+    def test_mod(self):
+        """test indentation of module / program"""
+        instring_mod = "module my_module\nintrinsic none\ncontains\nfunction my_func()\nend\nend module"
+        instring_prog = "program my_program\nintrinsic none\ncontains\nfunction my_func()\nend\nend program"
+
+        outstring_mod = "module my_module\n   intrinsic none\ncontains\n   function my_func()\n   end\nend module"
+        outstring_mod_disable = "module my_module\nintrinsic none\ncontains\nfunction my_func()\nend\nend module"
+
+        outstring_prog = "program my_program\n   intrinsic none\ncontains\n   function my_func()\n   end\nend program"
+        outstring_prog_disable = "program my_program\nintrinsic none\ncontains\nfunction my_func()\nend\nend program"
+
+        self.assert_fprettify_result([], instring_mod, outstring_mod)
+        self.assert_fprettify_result([], instring_prog, outstring_prog)
+
+        self.assert_fprettify_result(['--disable-indent-mod'], instring_mod, outstring_mod_disable)
+        self.assert_fprettify_result(['--disable-indent-mod'], instring_prog, outstring_prog_disable)
+
+    def test_decl(self):
+        """test formatting of declarations"""
+        instring_1 = "integer    ::     a"
+        instring_2 = "integer, dimension(:)    ::     a"
+        outstring_1 = "integer :: a"
+        outstring_2 = "integer, dimension(:) :: a"
+        outstring_2_min = "integer, dimension(:)::a"
+
+        self.assert_fprettify_result([], instring_1, instring_1)
+        self.assert_fprettify_result([], instring_2, instring_2)
+        self.assert_fprettify_result(['--enable-decl'], instring_1, outstring_1)
+        self.assert_fprettify_result(['--enable-decl'], instring_2, outstring_2)
+        self.assert_fprettify_result(['--enable-decl', '--whitespace-decl=0'], instring_2, outstring_2_min)
+
+    def test_statement_label(self):
+        instring = "1003  FORMAT(2(1x, i4), 5x, '-', 5x, '-', 3x, '-', 5x, '-', 5x, '-', 8x, '-', 3x, &\n    1p, 2(1x, d10.3))"
+        outstring = "1003 FORMAT(2(1x, i4), 5x, '-', 5x, '-', 3x, '-', 5x, '-', 5x, '-', 8x, '-', 3x, &\n            1p, 2(1x, d10.3))"
+        self.assert_fprettify_result([], instring, outstring)
+
+    def test_multiline_str(self):
+
+        instring = []
+        outstring = []
+
+        instring += [
+'''
+      CHARACTER(len=*), PARAMETER      :: serialized_string = &
+         "qtb_rng_gaussian                         1 F T F   0.0000000000000000E+00&
+                          12.0                12.0                12.0&
+                          12.0                12.0                12.0&
+                          12.0                12.0                12.0&
+                          12.0                12.0                12.0&
+                          12.0                12.0                12.0&
+                          12.0                12.0                12.0"
+'''
+]
+
+        outstring += [
+'''
+      CHARACTER(len=*), PARAMETER      :: serialized_string = &
+         "qtb_rng_gaussian                         1 F T F   0.0000000000000000E+00&
+&                          12.0                12.0                12.0&
+&                          12.0                12.0                12.0&
+&                          12.0                12.0                12.0&
+&                          12.0                12.0                12.0&
+&                          12.0                12.0                12.0&
+&                          12.0                12.0                12.0"
+'''
+]
+
+        instring += [
+'''
+      CHARACTER(len=*), PARAMETER      :: serialized_string = &
+         "qtb_rng_gaussian                         1 F T F   0.0000000000000000E+00&
+                 &         12.0                12.0                12.0&
+                 &         12.0                12.0                12.0&
+                 &         12.0                12.0                12.0&
+                 &         12.0                12.0                12.0&
+                 &         12.0                12.0                12.0&
+                 &         12.0                12.0                12.0"
+'''
+]
+
+        outstring += [
+'''
+      CHARACTER(len=*), PARAMETER      :: serialized_string = &
+         "qtb_rng_gaussian                         1 F T F   0.0000000000000000E+00&
+                 &         12.0                12.0                12.0&
+                 &         12.0                12.0                12.0&
+                 &         12.0                12.0                12.0&
+                 &         12.0                12.0                12.0&
+                 &         12.0                12.0                12.0&
+                 &         12.0                12.0                12.0"
+'''
+]
+
+        for instr, outstr in zip(instring, outstring):
+            self.assert_fprettify_result([], instr, outstr)
+
+
 
 def addtestmethod(testcase, fpath, ffile):
     """add a test method for each example."""
@@ -516,12 +932,7 @@ def addtestmethod(testcase, fpath, ffile):
 
     # not sure why this even works, using "test something" (with a space) as function name...
     # however it gives optimal test output
-    try:
-        testmethod.__name__ = ("test " + joinpath(fpath, ffile))
-    except TypeError: # pragma: no cover
-        # need to encode in python 2 since we are using unicode strings
-        testmethod.__name__ = (
-            "test " + joinpath(fpath, ffile)).encode('utf-8')
+    testmethod.__name__ = ("test " + joinpath(fpath, ffile))
 
     setattr(testcase, testmethod.__name__, testmethod)
 
