@@ -1,6 +1,4 @@
-#!/usr/bin/env python
-
-from __future__ import absolute_import
+#!/usr/bin/env python3
 
 import unittest
 import os
@@ -18,7 +16,7 @@ class TestSingleFileFolder(unittest.TestCase):
         self.fname = os.path.join(self.tempdir, "prettify_selftest.F")
 
         # create temporary file with example code
-        with open(self.fname, "w") as fhandle:
+        with open(self.fname, "w", encoding="utf8") as fhandle:
             fhandle.write(selftest.content)
 
     def tearDown(self):
@@ -30,7 +28,7 @@ class TestSingleFileFolder(unittest.TestCase):
         self.assertEqual(main([self.fname]), 0)
 
         # check if file was altered (it shouldn't)
-        with open(self.fname) as fhandle:
+        with open(self.fname, encoding="utf8") as fhandle:
             result = fhandle.read()
 
         self.assertEqual(result.splitlines(), selftest.content.splitlines())
