@@ -21,7 +21,7 @@ try:
     from typing import Literal  # not available before Python 3.8
 
     TestStatus = Literal["OK", "WRONG RESULT", "RUNTIME FAIL", "TIMED OUT"]
-except:
+except ImportError:
     TestStatus = str  # type: ignore
 
 # Some tests do not work with --keepalive (which is generally considered a bug).
@@ -32,6 +32,11 @@ KEEPALIVE_SKIP_DIRS = [
     "LIBTEST/libvori",
     "LIBTEST/libbqb",
     "QS/regtest-ec",
+    "QS/regtest-gapw",
+    "QS/regtest-gapw_xc",
+    "QS/regtest-gpw-1",
+    "QS/regtest-mp2-grad-1",
+    "QS/regtest-mp2-grad-2",
 ]
 
 # ======================================================================================
@@ -40,12 +45,14 @@ async def main() -> None:
     parser.add_argument("--mpiranks", type=int, default=2)
     parser.add_argument("--ompthreads", type=int)
     parser.add_argument("--maxtasks", type=int, default=os.cpu_count())
+    parser.add_argument("--num_gpus", type=int, default=0)
     parser.add_argument("--timeout", type=int, default=400)
     parser.add_argument("--maxerrors", type=int, default=50)
     parser.add_argument("--mpiexec", default="mpiexec")
     parser.add_argument("--keepalive", dest="keepalive", action="store_true")
     parser.add_argument("--debug", action="store_true")
     parser.add_argument("--restrictdir", action="append")
+    parser.add_argument("--skipdir", action="append")
     parser.add_argument("--workbasedir", type=Path)
     parser.add_argument("arch")
     parser.add_argument("version")
@@ -117,23 +124,28 @@ async def main() -> None:
 
     # Create async tasks.
     tasks = []
-    num_restrictdirs = 0
+    num_restrictdirs = num_skipdirs = 0
     for batch in batches:
         if not batch.requirements_satisfied(flags, cfg.mpiranks):
             print(f"Skipping {batch.name} because its requirements are not satisfied.")
         elif not any(re.fullmatch(p, batch.name) for p in cfg.restrictdirs):
             num_restrictdirs += 1
+        elif any(re.fullmatch(p, batch.name) for p in cfg.skipdirs):
+            num_skipdirs += 1
         else:
             tasks.append(asyncio.get_event_loop().create_task(run_batch(batch, cfg)))
 
     if num_restrictdirs:
         print(f"Skipping {num_restrictdirs} test directories because of --restrictdir.")
+    if num_skipdirs:
+        print(f"Skipping {num_skipdirs} test directories because of --skipdir.")
     if not tasks:
         print("\nNo test directories selected, check --restrictdir filter.")
         sys.exit(1)
+    print(f"Launched {len(tasks)} test directories and {cfg.num_workers} worker...\n")
+    sys.stdout.flush()
 
     # Wait for tasks to finish and print their results.
-    print(f"Launched {len(tasks)} test directories and {cfg.num_workers} worker...\n")
     all_results: List[TestResult] = []
     with open(cfg.error_summary, "wt", encoding="utf8", errors="replace") as err_fh:
         for num_done, task in enumerate(asyncio.as_completed(tasks)):
@@ -200,6 +212,7 @@ class Config:
         self.debug = args.debug
         self.max_errors = args.maxerrors
         self.restrictdirs = args.restrictdir if args.restrictdir else [".*"]
+        self.skipdirs = args.skipdir if args.skipdir else []
         datestamp = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
         leaf_dir = f"TEST-{args.arch}-{args.version}-{datestamp}"
         self.work_base_dir = (
@@ -213,12 +226,15 @@ class Config:
             # capture_output argument not available before Python 3.7
             return subprocess.run(cmd, shell=True, stdout=PIPE, stderr=DEVNULL).stdout
 
-        # Detect number of GPU devices.
-        nv_cmd = "nvidia-smi --query-gpu=gpu_name --format=csv,noheader | wc -l"
-        nv_gpus = int(run_with_capture_stdout(nv_cmd))
-        amd_cmd = "rocm-smi --showid --csv | grep card | wc -l"
-        amd_gpus = int(run_with_capture_stdout(amd_cmd))
-        self.num_gpus = nv_gpus + amd_gpus
+        # Detect number of GPU devices, if not specified by the user
+        if args.num_gpus > 0:
+            self.num_gpus = args.num_gpus
+        else:
+            nv_cmd = "nvidia-smi --query-gpu=gpu_name --format=csv,noheader | wc -l"
+            nv_gpus = int(run_with_capture_stdout(nv_cmd))
+            amd_cmd = "rocm-smi --showid --csv | grep card | wc -l"
+            amd_gpus = int(run_with_capture_stdout(amd_cmd))
+            self.num_gpus = nv_gpus + amd_gpus
         self.next_gpu = 0  # Used to assign devices round robin to processes.
 
     def launch_exe(
@@ -244,7 +260,7 @@ class Config:
 
 # ======================================================================================
 class TestType:
-    """ Search pattern for result values, ie. a line in the TEST_TYPES file. """
+    """Search pattern for result values, ie. a line in the TEST_TYPES file."""
 
     def __init__(self, line: str):
         parts = line.rsplit("!", 1)
@@ -254,7 +270,7 @@ class TestType:
             self.pattern = self.pattern.replace(c, f"\\{c}")  # escape special chars
         try:
             self.regex = re.compile(self.pattern)
-        except:
+        except Exception:
             print("Bad regex: " + self.pattern)
             raise
         self.column = int(parts[1])
@@ -271,7 +287,7 @@ class TestType:
 
 # ======================================================================================
 class Unittest:
-    """ A unit test, ie. a standalone binary that matches '*_unittest.{cfg.version}'."""
+    """A unit test, ie. a standalone binary that matches '*_unittest.{cfg.version}'."""
 
     def __init__(self, name: str, workdir: Path):
         self.name = name
@@ -280,7 +296,7 @@ class Unittest:
 
 # ======================================================================================
 class Regtest:
-    """ A single input file to test, ie. a line in a TEST_FILES file. """
+    """A single input file to test, ie. a line in a TEST_FILES file."""
 
     def __init__(self, line: str, test_types: List[Optional[TestType]], workdir: Path):
         parts = line.split()
@@ -297,7 +313,7 @@ class Regtest:
 
 # ======================================================================================
 class Batch:
-    """ A directory of tests, ie. a line in the TEST_DIRS file. """
+    """A directory of tests, ie. a line in the TEST_DIRS file."""
 
     def __init__(self, line: str, cfg: Config):
         parts = line.split()
@@ -322,15 +338,18 @@ class TestResult:
         test: Union[Regtest, Unittest],
         duration: float,
         status: TestStatus,
+        value: Optional[float] = None,
         error: Optional[str] = None,
     ):
         self.test = test
         self.duration = duration
         self.status = status
+        self.value = value
         self.error = error
 
     def __str__(self) -> str:
-        return f"    {self.test.name :<50s} {self.status :>30s} ( {self.duration:6.2f} sec)"
+        value = f"{self.value:.10g}" if self.value else "-"
+        return f"    {self.test.name :<80s} {value :>17} {self.status :>12s} ( {self.duration:6.2f} sec)"
 
 
 # ======================================================================================
@@ -369,18 +388,19 @@ class Cp2kShell:
     async def sendline(self, line: str) -> None:
         if self.cfg.debug:
             print("Sending: " + line)
-        assert len(line) < 80  # input buffer size
+        assert len(line) < 1024  # input buffer size
         assert self._child and self._child.stdin
         data = (line + "\n").encode("utf8")
         self._child.stdin.write(data)
         try:
-            await self._child.stdin.drain()
+            await asyncio.wait_for(self._child.stdin.drain(), timeout=self.cfg.timeout)
         except ConnectionResetError:
             pass
 
     async def readline(self) -> str:
         assert self._child and self._child.stdout
-        data = await self._child.stdout.readline()
+        data_future = self._child.stdout.readline()
+        data = await asyncio.wait_for(data_future, timeout=self.cfg.timeout)
         line = data.decode("utf8", errors="replace")
         if self.cfg.debug:
             print("Received: " + line, end="")
@@ -444,10 +464,10 @@ async def run_unittests(batch: Batch, cfg: Config) -> List[TestResult]:
         error = "x" * 100 + f"\n{test.out_path}\n{output_tail}\n\n"
         if timed_out:
             error += f"Timed out after {duration} seconds."
-            results.append(TestResult(test, duration, "TIMED OUT", error))
+            results.append(TestResult(test, duration, "TIMED OUT", error=error))
         elif returncode != 0:
             error += f"Runtime failure with code {returncode}."
-            results.append(TestResult(test, duration, "RUNTIME FAIL", error))
+            results.append(TestResult(test, duration, "RUNTIME FAIL", error=error))
         else:
             results.append(TestResult(test, duration, "OK"))
 
@@ -456,7 +476,7 @@ async def run_unittests(batch: Batch, cfg: Config) -> List[TestResult]:
 
 # ======================================================================================
 async def run_regtests(batch: Batch, cfg: Config) -> List[TestResult]:
-    if cfg.keepalive and not batch.name in KEEPALIVE_SKIP_DIRS:
+    if cfg.keepalive and batch.name not in KEEPALIVE_SKIP_DIRS:
         return await run_regtests_keepalive(batch, cfg)
     else:
         return await run_regtests_classic(batch, cfg)
@@ -517,27 +537,28 @@ def eval_regtest(
     error = "x" * 100 + f"\n{test.out_path}\n"
     if timed_out:
         error += f"{output_tail}\n\nTimed out after {duration} seconds."
-        return TestResult(test, duration, "TIMED OUT", error)
+        return TestResult(test, duration, "TIMED OUT", error=error)
     elif returncode != 0:
         error += f"{output_tail}\n\nRuntime failure with code {returncode}."
-        return TestResult(test, duration, "RUNTIME FAIL", error)
+        return TestResult(test, duration, "RUNTIME FAIL", error=error)
     elif not test.test_type:
         return TestResult(test, duration, "OK")  # test type zero
     else:
         value_txt = test.test_type.grep(output)
         if not value_txt:
             error += f"{output_tail}\n\nResult not found: '{test.test_type.pattern}'."
-            return TestResult(test, duration, "WRONG RESULT", error)
+            return TestResult(test, duration, "WRONG RESULT", error=error)
         else:
             # compare result to reference
-            diff = float(value_txt) - test.ref_value
+            value = float(value_txt)
+            diff = value - test.ref_value
             rel_error = abs(diff / test.ref_value if test.ref_value != 0.0 else diff)
             if rel_error <= test.tolerance:
-                return TestResult(test, duration, "OK")
+                return TestResult(test, duration, "OK", value)
             else:
                 error += f"Difference too large: {rel_error:.2e} > {test.tolerance}, "
                 error += f"ref_value: {test.ref_value_txt}, value: {value_txt}."
-                return TestResult(test, duration, "WRONG RESULT", error)
+                return TestResult(test, duration, "WRONG RESULT", value, error)
 
 
 # ======================================================================================
