@@ -54,6 +54,18 @@ if [[ "${git_sha}" != "<N/A>" ]] && command -v git > /dev/null 2>&1; then
     "${git_sha}"
 fi
 
+echo "------------------------- System information -----------------------------"
+uname -mrs
+case "$(uname -s)" in
+  Linux)
+    ldd --version | head -n 1
+    ;;
+  Darwin)
+    sw_vers
+    ;;
+esac
+echo ""
+
 echo "--------------------------- Resource limits ------------------------------"
 case "$(uname -s)" in
   Linux)
@@ -63,6 +75,7 @@ case "$(uname -s)" in
     launchctl limit
     ;;
 esac
+echo ""
 
 echo "--------------------------- SELinux --------------------------------------"
 if [[ -f /usr/sbin/getenforce ]]; then
@@ -70,13 +83,29 @@ if [[ -f /usr/sbin/getenforce ]]; then
 else
   echo "No SELinux installation found"
 fi
+echo ""
 
 echo "--------------------------- ARCH-file ------------------------------------"
 cat "./arch/${ARCH}.${VERSION}"
+
+if [ -x "$(command -v nvidia-smi)" ]; then
+  echo "--------------------------- NVIDIA-SMI -----------------------------------"
+  nvidia-smi
+  echo ""
+fi
+
+if [ -x "$(command -v rocm-smi)" ]; then
+  echo "--------------------------- ROCm-SMI -------------------------------------"
+  rocm-smi
+  echo ""
+fi
+
 echo "-------------------------- Build-Tools -----------------------------------"
 make toolversions ARCH="${ARCH}" VERSION="${VERSION}"
+
 echo "----------------------- External Modules ---------------------------------"
 make extversions ARCH="${ARCH}" VERSION="${VERSION}"
+
 echo "---------------------------- Modules -------------------------------------"
 if [ "$(type -t module)" = 'function' ]; then
   module list 2>&1
