@@ -99,7 +99,8 @@ OPTIONS:
                           or --with-openblas options will switch --math-mode to the
                           respective modes.
 --gpu-ver                 Selects the GPU architecture for which to compile. Available
-                          options are: K20X, K40, K80, P100, V100, Mi50, Mi100, and no.
+                          options are: K20X, K40, K80, P100, V100, Mi50, Mi100, Mi250, 
+                          and no.
                           This setting determines the value of nvcc's '-arch' flag.
                           Default = no.
 --libint-lmax             Maximum supported angular momentum by libint.
@@ -107,8 +108,8 @@ OPTIONS:
                           Default = 5
 --log-lines               Number of log file lines dumped in case of a non-zero exit code.
                           Default = 200
---generic                 Compile for a generic target system, i.e. do not optimize for
-                          the actual host system.
+--target-cpu              Compile for the specified target CPU (e.g. haswell or generic), i.e.
+                          do not optimize for the actual host system which is the default (native)
 --no-arch-files           Do not generate arch files
 --dry-run                 Write only config files, but don't actually build packages.
 
@@ -118,14 +119,6 @@ The --enable-FEATURE options follow the rules:
   --enable-FEATURE        The option keyword alone is equivalent to
                           --enable-FEATURE=yes
 
-  --enable-gcc-master     If you are installing GCC using this script
-                          this option forces the master development version
-                          to be installed.
-                          Default = no
-  --enable-libxsmm-master If you are installing libxsmm using this script
-                          this option forces the master development version
-                          to be installed.
-                          Default = no
   --enable-cuda           Turn on GPU (CUDA) support (can be combined
                           with --enable-opencl).
                           Default = no
@@ -159,11 +152,13 @@ The --with-PKG options follow the rules:
                           Default = system
   --with-cmake            Cmake utilities
                           Default = install
-  --with-openmpi          OpenMPI, important if you want parallel version of CP2K.
+  --with-openmpi          OpenMPI, important if you want a parallel version of CP2K.
                           Default = system
   --with-mpich            MPICH, MPI library like OpenMPI. one should
                           use only one of OpenMPI, MPICH or Intel MPI.
                           Default = system
+  --with-mpich-device     Select the MPICH device, implies the use of MPICH as MPI library
+                          Default = ch3
   --with-intelmpi         Intel MPI, MPI library like OpenMPI. one should
                           use only one of OpenMPI, MPICH or Intel MPI.
                           Default = system
@@ -223,6 +218,8 @@ The --with-PKG options follow the rules:
                           Default = install
   --with-libvori          Enable libvori for the Voronoi integration (and the BQB compressed trajectory format)
                           Default = install
+  --with-libtorch         Enable libtorch the machine learning framework needed for NequIP.
+                          Default = no
 
 FURTHER INSTRUCTIONS
 
@@ -260,7 +257,7 @@ mpi_list="mpich openmpi intelmpi"
 math_list="mkl acml openblas"
 lib_list="fftw libint libxc libxsmm cosma scalapack elpa plumed \
           spfft spla ptscotch superlu pexsi quip gsl spglib hdf5 libvdwxc sirius
-          libvori"
+          libvori libtorch"
 package_list="${tool_list} ${mpi_list} ${math_list} ${lib_list}"
 # ------------------------------------------------------------------------
 
@@ -306,8 +303,9 @@ with_spfft="__DONTUSE__"
 with_spla="__DONTUSE__"
 with_cosma="__INSTALL__"
 with_libvori="__INSTALL__"
+with_libtorch="__DONTUSE__"
 # for MPI, we try to detect system MPI variant
-if (command -v mpirun >&- 2>&-); then
+if (command -v mpirun > /dev/null 2>&1); then
   # check if we are dealing with openmpi, mpich or intelmpi
   if (mpirun --version 2>&1 | grep -s -q "HYDRA"); then
     echo "MPI is detected and it appears to be MPICH"
@@ -336,14 +334,13 @@ fi
 # default enable options
 dry_run="__FALSE__"
 no_arch_files="__FALSE__"
-export generic="__FALSE__"
 enable_tsan="__FALSE__"
-enable_gcc_master="__FALSE__"
-enable_libxsmm_master="__FALSE__"
 enable_opencl="__FALSE__"
 enable_cuda="__FALSE__"
 enable_hip="__FALSE__"
 export GPUVER="no"
+export MPICH_DEVICE="ch3"
+export TARGET_CPU="native"
 
 # default for libint
 export LIBINT_LMAX="5"
@@ -376,8 +373,20 @@ fi
 while [ $# -ge 1 ]; do
   case ${1} in
     -j)
-      shift
-      export NPROCS_OVERWRITE="${1}"
+      case "${2}" in
+        -*)
+          export NPROCS_OVERWRITE="$(get_nprocs)"
+          ;;
+        [0-9]*)
+          shift
+          export NPROCS_OVERWRITE="${1}"
+          ;;
+        *)
+          report_error ${LINENO} \
+            "The -j flag can only be followed by an integer number, found ${2}."
+          exit 1
+          ;;
+      esac
       ;;
     -j[0-9]*)
       export NPROCS_OVERWRITE="${1#-j}"
@@ -441,16 +450,20 @@ while [ $# -ge 1 ]; do
     --gpu-ver=*)
       user_input="${1#*=}"
       case "${user_input}" in
-        K20X | K40 | K80 | P100 | V100 | A100 | Mi50 | Mi100 | no)
+        K20X | K40 | K80 | P100 | V100 | A100 | Mi50 | Mi100 | Mi250 | no)
           export GPUVER="${user_input}"
           ;;
         *)
           report_error ${LINENO} \
-            "--gpu-ver currently only supports K20X, K40, K80, P100, V100, A100, Mi50, Mi100, and no as options"
+            "--gpu-ver currently only supports K20X, K40, K80, P100, V100, A100, Mi50, Mi100, Mi250, and no as options"
           exit 1
           ;;
       esac
       ;;
+    --target-cpu=*)
+      user_input="${1#*=}"
+      export TARGET_CPU="${user_input}"
+      ;;
     --log-lines=*)
       user_input="${1#*=}"
       export LOG_LINES="${user_input}"
@@ -462,9 +475,6 @@ while [ $# -ge 1 ]; do
     --no-arch-files)
       no_arch_files="__TRUE__"
       ;;
-    --generic)
-      export generic="__TRUE__"
-      ;;
     --dry-run)
       dry_run="__TRUE__"
       ;;
@@ -475,20 +485,6 @@ while [ $# -ge 1 ]; do
         exit 1
       fi
       ;;
-    --enable-gcc-master*)
-      enable_gcc_master=$(read_enable $1)
-      if [ "${enable_gcc_master}" = "__INVALID__" ]; then
-        report_error "invalid value for --enable-gcc-master, please use yes or no"
-        exit 1
-      fi
-      ;;
-    --enable-libxsmm-master*)
-      enable_libxsmm_master=$(read_enable $1)
-      if [ "${enable_libxsmm_master}" = "__INVALID__" ]; then
-        report_error "invalid value for --enable-libxsmm-master, please use yes or no"
-        exit 1
-      fi
-      ;;
     --enable-cuda*)
       enable_cuda=$(read_enable $1)
       if [ $enable_cuda = "__INVALID__" ]; then
@@ -523,6 +519,11 @@ while [ $# -ge 1 ]; do
     --with-cmake*)
       with_cmake=$(read_with "${1}")
       ;;
+    --with-mpich-device=*)
+      user_input="${1#*=}"
+      export MPICH_DEVICE="${user_input}"
+      export MPI_MODE=mpich
+      ;;
     --with-mpich*)
       with_mpich=$(read_with "${1}")
       if [ "${with_mpich}" != "__DONTUSE__" ]; then
@@ -619,6 +620,9 @@ while [ $# -ge 1 ]; do
     --with-libvori*)
       with_libvori=$(read_with "${1}")
       ;;
+    --with-libtorch*)
+      with_libtorch=$(read_with "${1}")
+      ;;
     --with-spla*)
       with_spla=$(read_with "${1}")
       ;;
@@ -644,8 +648,6 @@ export ENABLE_CUDA="${enable_cuda}"
 export ENABLE_HIP="${enable_hip}"
 export ENABLE_OPENCL="${enable_opencl}"
 export ENABLE_CRAY="${enable_cray}"
-[ "${enable_gcc_master}" = "__TRUE__" ] && export gcc_ver="master"
-[ "${enable_libxsmm_master}" = "__TRUE__" ] && export libxsmm_ver="master"
 
 # ------------------------------------------------------------------------
 # Check and solve known conflicts before installations proceed
@@ -781,8 +783,8 @@ if [ "${with_sirius}" = "__INSTALL__" ]; then
   [ "${with_fftw}" = "__DONTUSE__" ] && with_fftw="__INSTALL__"
   [ "${with_spglib}" = "__DONTUSE__" ] && with_spglib="__INSTALL__"
   [ "${with_hdf5}" = "__DONTUSE__" ] && with_hdf5="__INSTALL__"
-#  [ "${with_libvdwxc}" = "__DONTUSE__" ] && with_libvdwxc="__INSTALL__"
-#  [ "${with_cosma}" = "__DONTUSE__" ] && with_cosma="__INSTALL__"
+  [ "${with_libvdwxc}" = "__DONTUSE__" ] && with_libvdwxc="__INSTALL__"
+  [ "${with_cosma}" = "__DONTUSE__" ] && with_cosma="__INSTALL__"
 fi
 
 if [ "${with_plumed}" = "__INSTALL__" ]; then
@@ -857,7 +859,7 @@ if [ "${ENABLE_CRAY}" = "__TRUE__" ]; then
         export MPI_CFLAGS
         export MPI_LDFLAGS
         export MPI_LIBS=" "
-        export CP_DFLAGS="${CP_DFLAGS} IF_MPI(-D__parallel -D__MPI_VERSION=3|)"
+        export CP_DFLAGS="${CP_DFLAGS} IF_MPI(-D__parallel|)"
       fi
       ;;
     openmpi)
@@ -867,7 +869,7 @@ if [ "${ENABLE_CRAY}" = "__TRUE__" ]; then
         export MPI_CFLAGS
         export MPI_LDFLAGS
         export MPI_LIBS="-lmpi -lmpi_cxx"
-        export CP_DFLAGS="${CP_DFLAGS} IF_MPI(-D__parallel -D__MPI_VERSION=3|)"
+        export CP_DFLAGS="${CP_DFLAGS} IF_MPI(-D__parallel|)"
       fi
       ;;
     intelmpi)
@@ -879,7 +881,7 @@ if [ "${ENABLE_CRAY}" = "__TRUE__" ]; then
         export MPI_CFLAGS
         export MPI_LDFLAGS
         export MPI_LIBS="-lmpi -lmpi_cxx"
-        export CP_DFLAGS="${CP_DFLAGS} IF_MPI(-D__parallel -D__MPI_VERSION=3|)"
+        export CP_DFLAGS="${CP_DFLAGS} IF_MPI(-D__parallel|)"
       fi
       ;;
   esac
@@ -896,7 +898,7 @@ fi
 # Installing tools required for building CP2K and associated libraries
 # ------------------------------------------------------------------------
 
-echo "Compiling with $(get_nprocs) processes."
+echo "Compiling with $(get_nprocs) processes for target ${TARGET_CPU}."
 
 # Select the correct compute number based on the GPU architecture
 case ${GPUVER} in
@@ -924,12 +926,15 @@ case ${GPUVER} in
   Mi100)
     # TODO: export ARCH_NUM=
     ;;
+  Mi250)
+    # TODO: export ARCH_NUM=
+    ;;
   no)
     export ARCH_NUM="no"
     ;;
   *)
     report_error ${LINENO} \
-      "--gpu-ver currently only supports K20X, K40, K80, P100, V100, A100, Mi50, Mi100, and no as options"
+      "--gpu-ver currently only supports K20X, K40, K80, P100, V100, A100, Mi50, Mi100, Mi250, and no as options"
     exit 1
     ;;
 esac
