@@ -34,11 +34,15 @@ LD_arch="IF_MPI(${MPIFC}|${FC})"
 
 # we always want good line information and backtraces
 if [ "${with_intel}" != "__DONTUSE__" ]; then
-  BASEFLAGS="-nofor-main -O2 -fopenmp -fp-model precise -funroll-loops -g -qopenmp-simd -traceback -xHost -wd279"
-elif [ "${generic}" = "__TRUE__" ]; then
-  BASEFLAGS="-fno-omit-frame-pointer -fopenmp -g -mtune=generic"
+  if [ "${TARGET_CPU}" = "native" ]; then
+    BASEFLAGS="-fopenmp -fp-model precise -g -nofor-main -qopenmp-simd -traceback -wd279 -xHost"
+  elif [ "${TARGET_CPU}" = "generic" ]; then
+    BASEFLAGS="-fopenmp -fp-model precise -g -mtune=${TARGET_CPU} -nofor-main -qopenmp-simd -traceback -wd279"
+  else
+    BASEFLAGS="-fopenmp -fp-model precise -g -march=${TARGET_CPU} -mtune=${TARGET_CPU} -nofor-main -qopenmp-simd -traceback -wd279"
+  fi
 else
-  BASEFLAGS="-fno-omit-frame-pointer -fopenmp -g -march=native -mtune=native"
+  BASEFLAGS="-fno-omit-frame-pointer -fopenmp -g -mtune=${TARGET_CPU} IF_ASAN(-fsanitize=address|)"
 fi
 
 OPT_FLAGS="-O3 -funroll-loops"
@@ -114,7 +118,7 @@ LDFLAGS="IF_STATIC(${STATIC_FLAGS}|) \$(FCFLAGS) ${CP_LDFLAGS}"
 # add standard libs
 LIBS="${CP_LIBS} -lstdc++"
 
-CXXFLAGS+=" --std=c++11 \$(DFLAGS) -Wno-deprecated-declarations"
+CXXFLAGS+=" --std=c++14 \$(DFLAGS) -Wno-deprecated-declarations"
 # CUDA handling
 if [ "${ENABLE_CUDA}" = __TRUE__ ] && [ "${GPUVER}" != no ]; then
   CUDA_LIBS="-lcudart -lnvrtc -lcuda -lcufft -lcublas -lrt IF_DEBUG(-lnvToolsExt|)"
@@ -173,7 +177,7 @@ if [ "${ENABLE_HIP}" = __TRUE__ ] && [ "${GPUVER}" != no ]; then
       HIP_FLAGS+="-fPIE -D__HIP_PLATFORM_AMD__ -g --offload-arch=gfx906 -O3 --std=c++11 -Wall -Wextra -Werror \$(DFLAGS)"
       LIBS+=" IF_HIP(-lamdhip64 -lhipfft -lhipblas -lrocblas IF_DEBUG(-lroctx64 -lroctracer64|)|)"
       DFLAGS+=" IF_HIP(-D__HIP_PLATFORM_AMD__ -D__OFFLOAD_HIP IF_DEBUG(-D__OFFLOAD_PROFILING|)|)  -D__DBCSR_ACC"
-      CXXFLAGS+=" -fopenmp -std=c++11 -Wall -Wextra -Werror"
+      CXXFLAGS+=" -fopenmp -Wall -Wextra -Werror"
       ;;
     Mi100)
       check_lib -lamdhip64 "hip"
@@ -189,7 +193,23 @@ if [ "${ENABLE_HIP}" = __TRUE__ ] && [ "${GPUVER}" != no ]; then
       HIP_FLAGS+="-fPIE -D__HIP_PLATFORM_AMD__ -g --offload-arch=gfx908 -O3 --std=c++11 -Wall -Wextra -Werror \$(DFLAGS)"
       LIBS+=" IF_HIP(-lamdhip64 -lhipfft -lhipblas -lrocblas IF_DEBUG(-lroctx64 -lroctracer64|)|)"
       DFLAGS+=" IF_HIP(-D__HIP_PLATFORM_AMD__ -D__OFFLOAD_HIP IF_DEBUG(-D__OFFLOAD_PROFILING|)|) -D__DBCSR_ACC"
-      CXXFLAGS+=" -fopenmp -std=c++11 -Wall -Wextra -Werror"
+      CXXFLAGS+=" -fopenmp -Wall -Wextra -Werror"
+      ;;
+    Mi250)
+      check_lib -lamdhip64 "hip"
+      add_lib_from_paths HIP_LDFLAGS "libamdhip64.*" $LIB_PATHS
+      check_lib -lhipfft "hip"
+      add_lib_from_paths HIP_LDFLAGS "libhipfft.*" $LIB_PATHS
+      check_lib -lrocblas "hip"
+      add_lib_from_paths HIP_LDFLAGS "librocblas.*" $LIB_PATHS
+      check_lib -lroctx64 "hip"
+      add_lib_from_paths HIP_LDFLAGS "libroctx64.*" $LIB_PATHS
+      check_lib -lroctracer64 "hip"
+      add_lib_from_paths HIP_LDFLAGS "libroctracer64.*" $LIB_PATHS
+      HIP_FLAGS+="-fPIE -D__HIP_PLATFORM_AMD__ -g --offload-arch=gfx90a -O3 --std=c++11 -Wall -Wextra -Werror \$(DFLAGS)"
+      LIBS+=" IF_HIP(-lamdhip64 -lhipfft -lhipblas -lrocblas IF_DEBUG(-lroctx64 -lroctracer64|)|)"
+      DFLAGS+=" IF_HIP(-D__HIP_PLATFORM_AMD__ -D__OFFLOAD_HIP IF_DEBUG(-D__OFFLOAD_PROFILING|)|) -D__DBCSR_ACC"
+      CXXFLAGS+=" -fopenmp -Wall -Wextra -Werror"
       ;;
     *)
       check_command nvcc "cuda"
@@ -199,7 +219,7 @@ if [ "${ENABLE_HIP}" = __TRUE__ ] && [ "${GPUVER}" != no ]; then
       check_lib -lcufft "cuda"
       check_lib -lcublas "cuda"
       DFLAGS+=" IF_HIP(-D__HIP_PLATFORM_NVIDIA__ -D__HIP_PLATFORM_NVCC__ -D__OFFLOAD_HIP |) -D__DBCSR_ACC"
-      HIP_FLAGS=" -g -arch sm_${ARCH_NUM} -O3 -Xcompiler='-fopenmp -Wall -Wextra -Werror' --std=c++11 \$(DFLAGS)"
+      HIP_FLAGS+=" -g -arch sm_${ARCH_NUM} -O3 -Xcompiler='-fopenmp -Wall -Wextra -Werror' --std=c++11 \$(DFLAGS)"
       add_include_from_paths CUDA_CFLAGS "cuda.h" $INCLUDE_PATHS
       HIP_INCLUDES+=" -I${CUDA_PATH:-${CUDA_HOME:-/CUDA_HOME-notset}}/include"
       # GCC issues lots of warnings for hip/nvidia_detail/hip_runtime_api.h
@@ -333,12 +353,14 @@ rm -f ${INSTALLDIR}/arch/local*
 gen_arch_file "local.ssmp"
 gen_arch_file "local_static.ssmp" STATIC
 gen_arch_file "local.sdbg" DEBUG
+gen_arch_file "local_asan.ssmp" ASAN
 gen_arch_file "local_coverage.sdbg" COVERAGE
 arch_vers="ssmp sdbg"
 
 if [ "$MPI_MODE" != no ]; then
   gen_arch_file "local.psmp" MPI
   gen_arch_file "local.pdbg" MPI DEBUG
+  gen_arch_file "local_asan.psmp" MPI ASAN
   gen_arch_file "local_static.psmp" MPI STATIC
   gen_arch_file "local_warn.psmp" MPI WARNALL
   gen_arch_file "local_coverage.pdbg" MPI COVERAGE
