@@ -6,8 +6,8 @@
 [ "${BASH_SOURCE[0]}" ] && SCRIPT_NAME="${BASH_SOURCE[0]}" || SCRIPT_NAME=$0
 SCRIPT_DIR="$(cd "$(dirname "$SCRIPT_NAME")/.." && pwd -P)"
 
-gcc_ver="12.1.0"
-gcc_sha256="e88a004a14697bbbaba311f38a938c716d9a652fd151aaaa4cf1b5b99b90e2de"
+gcc_ver="12.2.0"
+gcc_sha256="ac6b317eb4d25444d87cf29c0d141dedc1323a1833ec9995211b13e1a851261c"
 
 source "${SCRIPT_DIR}"/common_vars.sh
 source "${SCRIPT_DIR}"/tool_kit.sh
@@ -31,21 +31,19 @@ case "${with_gcc}" in
     if verify_checksums "${install_lock_file}"; then
       echo "gcc-${gcc_ver} is already installed, skipping it."
     else
-      if [ "${gcc_ver}" = "master" ]; then
-        [ -d gcc-master ] && rm -rf gcc-master
-        svn checkout svn://gcc.gnu.org/svn/gcc/trunk gcc-master > svn-gcc.log 2>&1 || tail -n ${LOG_LINES} svn-gcc.log
+      if [ -f gcc-${gcc_ver}.tar.gz ]; then
+        echo "gcc-${gcc_ver}.tar.gz is found"
       else
-        if [ -f gcc-${gcc_ver}.tar.gz ]; then
-          echo "gcc-${gcc_ver}.tar.gz is found"
-        else
-          download_pkg ${DOWNLOADER_FLAGS} ${gcc_sha256} \
-            "https://www.cp2k.org/static/downloads/gcc-${gcc_ver}.tar.gz"
-        fi
-        [ -d gcc-${gcc_ver} ] && rm -rf gcc-${gcc_ver}
-        tar -xzf gcc-${gcc_ver}.tar.gz
+        download_pkg_from_cp2k_org "${gcc_sha256}" "gcc-${gcc_ver}.tar.gz"
       fi
+      [ -d gcc-${gcc_ver} ] && rm -rf gcc-${gcc_ver}
+      tar -xzf gcc-${gcc_ver}.tar.gz
+
       echo "Installing GCC from scratch into ${pkg_install_dir}"
       cd gcc-${gcc_ver}
+
+      # Download prerequisites from cp2k.org because gcc.gnu.org returns 403 when queried from GCP.
+      sed -i 's|http://gcc.gnu.org/pub/gcc/infrastructure/|https://cp2k.org/static/downloads/|' ./contrib/download_prerequisites
       ./contrib/download_prerequisites > prereq.log 2>&1 || tail -n ${LOG_LINES} prereq.log
       GCCROOT=${PWD}
       mkdir obj
@@ -117,7 +115,7 @@ case "${with_gcc}" in
     F90="${FC}"
     F77="${FC}"
     GCC_CFLAGS="-I'${pkg_install_dir}/include'"
-    GCC_LDFLAGS="-L'${pkg_install_dir}/lib64' -L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib64' -Wl,-rpath='${pkg_install_dir}/lib64'"
+    GCC_LDFLAGS="-L'${pkg_install_dir}/lib64' -L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib64' -Wl,-rpath,'${pkg_install_dir}/lib64'"
     ;;
   __SYSTEM__)
     echo "==================== Finding GCC from system paths ===================="
@@ -145,7 +143,7 @@ case "${with_gcc}" in
     F90="${FC}"
     F77="${FC}"
     GCC_CFLAGS="-I'${pkg_install_dir}/include'"
-    GCC_LDFLAGS="-L'${pkg_install_dir}/lib64' -L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib64' -Wl,-rpath='${pkg_install_dir}/lib64'"
+    GCC_LDFLAGS="-L'${pkg_install_dir}/lib64' -L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib64' -Wl,-rpath,'${pkg_install_dir}/lib64'"
     ;;
 esac
 if [ "${ENABLE_TSAN}" = "__TRUE__" ]; then
