@@ -6,8 +6,8 @@
 [ "${BASH_SOURCE[0]}" ] && SCRIPT_NAME="${BASH_SOURCE[0]}" || SCRIPT_NAME=$0
 SCRIPT_DIR="$(cd "$(dirname "$SCRIPT_NAME")/.." && pwd -P)"
 
-mpich_ver="3.4.3"
-mpich_sha256="8154d89f3051903181018166678018155f4c2b6f04a9bb6fe9515656452c4fd7"
+mpich_ver="4.0.3"
+mpich_sha256="17406ea90a6ed4ecd5be39c9ddcbfac9343e6ab4f77ac4e8c5ebe4a3e3b6c501"
 mpich_pkg="mpich-${mpich_ver}.tar.gz"
 
 source "${SCRIPT_DIR}"/common_vars.sh
@@ -36,10 +36,9 @@ case "${with_mpich}" in
       if [ -f ${mpich_pkg} ]; then
         echo "${mpich_pkg} is found"
       else
-        download_pkg ${DOWNLOADER_FLAGS} ${mpich_sha256} \
-          https://www.cp2k.org/static/downloads/${mpich_pkg}
+        download_pkg_from_cp2k_org "${mpich_sha256}" "${mpich_pkg}"
       fi
-      echo "Installing from scratch into ${pkg_install_dir}"
+      echo "Installing from scratch into ${pkg_install_dir} for MPICH device ${MPICH_DEVICE}"
       [ -d mpich-${mpich_ver} ] && rm -rf mpich-${mpich_ver}
       tar -xzf ${mpich_pkg}
       cd mpich-${mpich_ver}
@@ -59,7 +58,7 @@ case "${with_mpich}" in
         FCFLAGS="${FCFLAGS} ${compat_flag}" \
         --without-x \
         --enable-gl=no \
-        --with-device=ch3 \
+        --with-device=${MPICH_DEVICE} \
         > configure.log 2>&1 || tail -n ${LOG_LINES} configure.log
       make -j $(get_nprocs) > make.log 2>&1 || tail -n ${LOG_LINES} make.log
       make install > install.log 2>&1 || tail -n ${LOG_LINES} install.log
@@ -76,13 +75,13 @@ case "${with_mpich}" in
     MPIF90="${MPIFC}"
     MPIF77="${MPIFC}"
     MPICH_CFLAGS="-I'${pkg_install_dir}/include'"
-    MPICH_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    MPICH_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
   __SYSTEM__)
     echo "==================== Finding MPICH from system paths ===================="
     check_command mpirun "mpich" && MPIRUN="$(command -v mpirun)" || exit 1
     check_command mpicc "mpich" && MPICC="$(command -v mpicc)" || exit 1
-    if [ $(command -v mpic++ >&- 2>&-) ]; then
+    if [ $(command -v mpic++ > /dev/null 2>&1) ]; then
       check_command mpic++ "mpich" && MPICXX="$(command -v mpic++)" || exit 1
     else
       check_command mpicxx "mpich" && MPICXX="$(command -v mpicxx)" || exit 1
@@ -112,7 +111,7 @@ case "${with_mpich}" in
     MPIF90="${MPIFC}"
     MPIF77="${MPIFC}"
     MPICH_CFLAGS="-I'${pkg_install_dir}/include'"
-    MPICH_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    MPICH_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
 esac
 if [ "${with_mpich}" != "__DONTUSE__" ]; then
@@ -121,15 +120,6 @@ if [ "${with_mpich}" != "__DONTUSE__" ]; then
   else
     mpi_bin="mpirun"
   fi
-  # check MPICH version, versions less than 3.0 will get -D__MPI_VERSION=2 flag
-  raw_version=$(${mpi_bin} --version | grep "Version:" | awk '{print $2}')
-  major_version=$(echo $raw_version | cut -d '.' -f 1)
-  minor_version=$(echo $raw_version | cut -d '.' -f 2)
-  if [ ${major_version} -lt 3 ]; then
-    mpi2_dflags="-D__MPI_VERSION=2"
-  else
-    mpi2_dflags=""
-  fi
   MPICH_LIBS="-lmpifort -lmpicxx -lmpi"
   cat << EOF > "${BUILDDIR}/setup_mpich"
 export MPI_MODE="${MPI_MODE}"
@@ -145,7 +135,7 @@ export MPICH_LIBS="${MPICH_LIBS}"
 export MPI_CFLAGS="${MPICH_CFLAGS}"
 export MPI_LDFLAGS="${MPICH_LDFLAGS}"
 export MPI_LIBS="${MPICH_LIBS}"
-export CP_DFLAGS="\${CP_DFLAGS} IF_MPI(-D__parallel ${mpi2_dflags}|)"
+export CP_DFLAGS="\${CP_DFLAGS} IF_MPI(-D__parallel|)"
 export CP_CFLAGS="\${CP_CFLAGS} IF_MPI(${MPICH_CFLAGS}|)"
 export CP_LDFLAGS="\${CP_LDFLAGS} IF_MPI(${MPICH_LDFLAGS}|)"
 export CP_LIBS="\${CP_LIBS} IF_MPI(${MPICH_LIBS}|)"
