@@ -6,8 +6,8 @@
 [ "${BASH_SOURCE[0]}" ] && SCRIPT_NAME="${BASH_SOURCE[0]}" || SCRIPT_NAME=$0
 SCRIPT_DIR="$(cd "$(dirname "$SCRIPT_NAME")/.." && pwd -P)"
 
-openmpi_ver="4.1.1"
-openmpi_sha256="d80b9219e80ea1f8bcfe5ad921bd9014285c4948c5965f4156a3831e60776444"
+openmpi_ver="4.1.4"
+openmpi_sha256="e166dbe876e13a50c2882e11193fecbc4362e89e6e7b6deeb69bf095c0f4fc4c"
 openmpi_pkg="openmpi-${openmpi_ver}.tar.gz"
 
 source "${SCRIPT_DIR}"/common_vars.sh
@@ -36,23 +36,24 @@ case "${with_openmpi}" in
       if [ -f ${openmpi_pkg} ]; then
         echo "${openmpi_pkg} is found"
       else
-        download_pkg ${DOWNLOADER_FLAGS} ${openmpi_sha256} \
-          "https://www.cp2k.org/static/downloads/${openmpi_pkg}"
+        download_pkg_from_cp2k_org "${openmpi_sha256}" "${openmpi_pkg}"
       fi
       echo "Installing from scratch into ${pkg_install_dir}"
       [ -d openmpi-${openmpi_ver} ] && rm -rf openmpi-${openmpi_ver}
       tar -xzf ${openmpi_pkg}
       cd openmpi-${openmpi_ver}
-      # can have issue with older glibc libraries, in which case
-      # we need to add the -fgnu89-inline to CFLAGS. We can check
-      # the version of glibc using ldd --version, as ldd is part of
-      # glibc package
-      glibc_version=$(ldd --version | awk '/ldd/{print $NF}')
-      glibc_major_ver=${glibc_version%%.*}
-      glibc_minor_ver=${glibc_version##*.}
-      if [ $glibc_major_ver -lt 2 ] ||
-        [ $glibc_major_ver -eq 2 -a $glibc_minor_ver -lt 12 ]; then
-        CFLAGS="${CFLAGS} -fgnu89-inline"
+      if [ "${OPENBLAS_ARCH}" = "x86_64" ]; then
+        # can have issue with older glibc libraries, in which case
+        # we need to add the -fgnu89-inline to CFLAGS. We can check
+        # the version of glibc using ldd --version, as ldd is part of
+        # glibc package
+        glibc_version=$(ldd --version | awk '/ldd/{print $NF}')
+        glibc_major_ver=${glibc_version%%.*}
+        glibc_minor_ver=${glibc_version##*.}
+        if [ $glibc_major_ver -lt 2 ] ||
+          [ $glibc_major_ver -eq 2 -a $glibc_minor_ver -lt 12 ]; then
+          CFLAGS="${CFLAGS} -fgnu89-inline"
+        fi
       fi
       if [ $(command -v srun) ]; then
         echo "Slurm installation found. OpenMPI will be configured with --with-pmi."
@@ -60,6 +61,7 @@ case "${with_openmpi}" in
       else
         EXTRA_CONFIGURE_FLAGS=""
       fi
+      # We still require MPI-1.0-compatability for PTSCOTCH
       ./configure CFLAGS="${CFLAGS}" \
         --prefix=${pkg_install_dir} \
         --libdir="${pkg_install_dir}/lib" \
@@ -81,7 +83,7 @@ case "${with_openmpi}" in
     MPIF90="${MPIFC}"
     MPIF77="${MPIFC}"
     OPENMPI_CFLAGS="-I'${pkg_install_dir}/include'"
-    OPENMPI_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    OPENMPI_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
   __SYSTEM__)
     echo "==================== Finding OpenMPI from system paths ===================="
@@ -112,7 +114,7 @@ case "${with_openmpi}" in
     MPIF90="${MPIFC}"
     MPIF77="${MPIFC}"
     OPENMPI_CFLAGS="-I'${pkg_install_dir}/include'"
-    OPENMPI_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    OPENMPI_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
 esac
 if [ "${with_openmpi}" != "__DONTUSE__" ]; then
@@ -135,13 +137,6 @@ if [ "${with_openmpi}" != "__DONTUSE__" ]; then
     OPENMPI_LIBS+=" -l${lib}"
     OPENMPI_LDFLAGS="${OPENMPI_LDFLAGS//-l${lib}/}"
   done
-  # old versions didn't support MPI 3, so adjust __MPI_VERSION accordingly (needed e.g. for pexsi)
-  if [[ "$major_version" =~ ^[0-9]+$ ]] && ([ $major_version -lt 1 ] ||
-    [ $major_version -eq 1 -a ${minor_version} -lt 7 ]); then
-    mpi2_dflags="-D__MPI_VERSION=2"
-  else
-    mpi2_dflags=""
-  fi
   cat << EOF > "${BUILDDIR}/setup_openmpi"
 export MPI_MODE="${MPI_MODE}"
 export MPIRUN="${MPIRUN}"
@@ -156,7 +151,7 @@ export OPENMPI_LIBS="${OPENMPI_LIBS}"
 export MPI_CFLAGS="${OPENMPI_CFLAGS}"
 export MPI_LDFLAGS="${OPENMPI_LDFLAGS}"
 export MPI_LIBS="${OPENMPI_LIBS}"
-export CP_DFLAGS="\${CP_DFLAGS} IF_MPI(-D__parallel ${mpi2_dflags}|)"
+export CP_DFLAGS="\${CP_DFLAGS} IF_MPI(-D__parallel|)"
 export CP_CFLAGS="\${CP_CFLAGS} IF_MPI(${OPENMPI_CFLAGS}|)"
 export CP_LDFLAGS="\${CP_LDFLAGS} IF_MPI(${OPENMPI_LDFLAGS}|)"
 export CP_LIBS="\${CP_LIBS} IF_MPI(${OPENMPI_LIBS}|)"
