@@ -6,8 +6,8 @@
 [ "${BASH_SOURCE[0]}" ] && SCRIPT_NAME="${BASH_SOURCE[0]}" || SCRIPT_NAME=$0
 SCRIPT_DIR="$(cd "$(dirname "$SCRIPT_NAME")/.." && pwd -P)"
 
-libxc_ver="5.2.3"
-libxc_sha256="7b7a96d8eeb472c7b8cca7ac38eae27e0a8113ef44dae5359b0eb12592b4bcf2"
+libxc_ver="6.0.0"
+libxc_sha256="c2ca205a762200dfba2e6c9e8ca2061aaddc6b7cf42048859fe717a7aa07de7c"
 source "${SCRIPT_DIR}"/common_vars.sh
 source "${SCRIPT_DIR}"/tool_kit.sh
 source "${SCRIPT_DIR}"/signal_trap.sh
@@ -33,13 +33,16 @@ case "$with_libxc" in
       if [ -f libxc-${libxc_ver}.tar.gz ]; then
         echo "libxc-${libxc_ver}.tar.gz is found"
       else
-        download_pkg ${DOWNLOADER_FLAGS} ${libxc_sha256} \
-          https://www.cp2k.org/static/downloads/libxc-${libxc_ver}.tar.gz
+        download_pkg_from_cp2k_org "${libxc_sha256}" "libxc-${libxc_ver}.tar.gz"
       fi
       echo "Installing from scratch into ${pkg_install_dir}"
       [ -d libxc-${libxc_ver} ] && rm -rf libxc-${libxc_ver}
       tar -xzf libxc-${libxc_ver}.tar.gz
       cd libxc-${libxc_ver}
+
+      # Fix missing comma at end of line in src/mgga_xc_b97mv.c
+      patch -p1 < "${SCRIPT_DIR}/stage3/libxc-${libxc_ver}_mgga_xc_b97mv.patch"
+
       # CP2K does not make use of fourth derivatives, so skip their compilation with --disable-lxc
       ./configure --prefix="${pkg_install_dir}" --libdir="${pkg_install_dir}/lib" --disable-lxc \
         > configure.log 2>&1 || tail -n ${LOG_LINES} configure.log
@@ -49,7 +52,7 @@ case "$with_libxc" in
       write_checksums "${install_lock_file}" "${SCRIPT_DIR}/stage3/$(basename ${SCRIPT_NAME})"
     fi
     LIBXC_CFLAGS="-I'${pkg_install_dir}/include'"
-    LIBXC_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    LIBXC_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
   __SYSTEM__)
     echo "==================== Finding LIBXC from system paths ===================="
@@ -66,7 +69,7 @@ case "$with_libxc" in
     check_dir "${pkg_install_dir}/lib"
     check_dir "${pkg_install_dir}/include"
     LIBXC_CFLAGS="-I'${pkg_install_dir}/include'"
-    LIBXC_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    LIBXC_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
 esac
 if [ "$with_libxc" != "__DONTUSE__" ]; then
