@@ -25,8 +25,7 @@ cd "${BUILDDIR}"
 case "$with_libxsmm" in
   __INSTALL__)
     echo "==================== Installing Libxsmm ===================="
-    if [[ ("$OPENBLAS_ARCH" != "x86_64") && (\
-      "$OPENBLAS_ARCH" != "arm64" || "$libxsmm_ver" != "master") ]]; then
+    if [[ ("$OPENBLAS_ARCH" != "x86_64") && ("$OPENBLAS_ARCH" != "arm64") ]]; then
       report_warning $LINENO "libxsmm is not supported on arch ${OPENBLAS_ARCH}"
       cat << EOF > "${BUILDDIR}/setup_libxsmm"
 with_libxsmm="__DONTUSE__"
@@ -38,22 +37,14 @@ EOF
     if verify_checksums "${install_lock_file}"; then
       echo "libxsmm-${libxsmm_ver} is already installed, skipping it."
     else
-      if [ "$libxsmm_ver" = "master" ]; then
-        download_pkg_no_checksum ${DOWNLOADER_FLAGS} \
-          -o libxsmm-master.zip \
-          https://github.com/hfp/libxsmm/archive/master.zip
-        [ -d libxsmm-master ] && rm -rf libxsmm-master
-        unzip -q -o libxsmm-master.zip
+      if [ -f libxsmm-${libxsmm_ver}.tar.gz ]; then
+        echo "libxsmm-${libxsmm_ver}.tar.gz is found"
       else
-        if [ -f libxsmm-${libxsmm_ver}.tar.gz ]; then
-          echo "libxsmm-${libxsmm_ver}.tar.gz is found"
-        else
-          download_pkg ${DOWNLOADER_FLAGS} ${libxsmm_sha256} \
-            https://www.cp2k.org/static/downloads/libxsmm-${libxsmm_ver}.tar.gz
-        fi
-        [ -d libxsmm-${libxsmm_ver} ] && rm -rf libxsmm-${libxsmm_ver}
-        tar -xzf libxsmm-${libxsmm_ver}.tar.gz
+        download_pkg_from_cp2k_org "${libxsmm_sha256}" "libxsmm-${libxsmm_ver}.tar.gz"
       fi
+      [ -d libxsmm-${libxsmm_ver} ] && rm -rf libxsmm-${libxsmm_ver}
+      tar -xzf libxsmm-${libxsmm_ver}.tar.gz
+
       echo "Installing from scratch into ${pkg_install_dir}"
       # note that we do not have to set -L flags to ld for the
       # linked math libraries for the libxsmm build, as for a
@@ -82,7 +73,7 @@ EOF
       write_checksums "${install_lock_file}" "${SCRIPT_DIR}/stage4/$(basename ${SCRIPT_NAME})"
     fi
     LIBXSMM_CFLAGS="-I'${pkg_install_dir}/include'"
-    LIBXSMM_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    LIBXSMM_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
   __SYSTEM__)
     echo "==================== Finding Libxsmm from system paths ===================="
@@ -100,7 +91,7 @@ EOF
     check_dir "${pkg_install_dir}/include"
     check_dir "${pkg_install_dir}/lib"
     LIBXSMM_CFLAGS="-I'${pkg_install_dir}/include'"
-    LIBXSMM_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    LIBXSMM_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
 esac
 if [ "$with_libxsmm" != "__DONTUSE__" ]; then
