@@ -6,12 +6,9 @@
 [ "${BASH_SOURCE[0]}" ] && SCRIPT_NAME="${BASH_SOURCE[0]}" || SCRIPT_NAME=$0
 SCRIPT_DIR="$(cd "$(dirname "$SCRIPT_NAME")/.." && pwd -P)"
 
-scalapack_ver="2.1.0"
-scalapack_sha256="61d9216cf81d246944720cfce96255878a3f85dec13b9351f1fa0fd6768220a6"
+scalapack_ver="2.2.1"
+scalapack_sha256="4aede775fdb28fa44b331875730bcd5bab130caaec225fadeccf424c8fcb55aa"
 scalapack_pkg="scalapack-${scalapack_ver}.tgz"
-patches=(
-  "${SCRIPT_DIR}/stage4/scalapack-${scalapack_ver}-gcc10.patch"
-)
 
 source "${SCRIPT_DIR}"/common_vars.sh
 source "${SCRIPT_DIR}"/tool_kit.sh
@@ -39,27 +36,26 @@ case "$with_scalapack" in
       if [ -f ${scalapack_pkg} ]; then
         echo "${scalapack_pkg} is found"
       else
-        download_pkg ${DOWNLOADER_FLAGS} ${scalapack_sha256} \
-          https://www.cp2k.org/static/downloads/${scalapack_pkg}
+        download_pkg_from_cp2k_org "${scalapack_sha256}" "${scalapack_pkg}"
       fi
       echo "Installing from scratch into ${pkg_install_dir}"
       [ -d scalapack-${scalapack_ver} ] && rm -rf scalapack-${scalapack_ver}
       tar -xzf ${scalapack_pkg}
 
-      pushd "scalapack-${scalapack_ver}" > /dev/null
-      for patch in "${patches[@]}"; do
-        patch -p1 < "${patch}" >> patch.log 2>&1
-      done
-      popd > /dev/null
-
       mkdir -p "scalapack-${scalapack_ver}/build"
       pushd "scalapack-${scalapack_ver}/build" > /dev/null
 
-      cmake -DCMAKE_FIND_ROOT_PATH="$ROOTDIR" \
+      flags=""
+      if ("${FC}" --version | grep -q 'GNU'); then
+        flags=$(allowed_gfortran_flags "-fallow-argument-mismatch")
+      fi
+      FFLAGS=$flags cmake -DCMAKE_FIND_ROOT_PATH="$ROOTDIR" \
         -DCMAKE_INSTALL_PREFIX="${pkg_install_dir}" \
         -DCMAKE_INSTALL_LIBDIR="lib" \
         -DBUILD_SHARED_LIBS=NO \
         -DCMAKE_BUILD_TYPE=Release .. \
+        -DBUILD_TESTING=NO \
+        -DSCALAPACK_BUILD_TESTS=NO \
         > configure.log 2>&1 || tail -n ${LOG_LINES} configure.log
       make -j $(get_nprocs) > make.log 2>&1 || tail -n ${LOG_LINES} make.log
       make install >> make.log 2>&1 || tail -n ${LOG_LINES} make.log
@@ -67,7 +63,7 @@ case "$with_scalapack" in
       popd > /dev/null
       write_checksums "${install_lock_file}" "${SCRIPT_DIR}/stage4/$(basename ${SCRIPT_NAME})"
     fi
-    SCALAPACK_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    SCALAPACK_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
   __SYSTEM__)
     echo "==================== Finding ScaLAPACK from system paths ===================="
@@ -80,7 +76,7 @@ case "$with_scalapack" in
     echo "==================== Linking ScaLAPACK to user paths ===================="
     pkg_install_dir="$with_scalapack"
     check_dir "${pkg_install_dir}/lib"
-    SCALAPACK_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    SCALAPACK_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
 esac
 if [ "$with_scalapack" != "__DONTUSE__" ]; then
