@@ -33,8 +33,7 @@ case "${with_ptscotch}" in
       if [ -f scotch_${scotch_ver}.tar.gz ]; then
         echo "scotch_${scotch_ver}.tar.gz is found"
       else
-        download_pkg ${DOWNLOADER_FLAGS} ${scotch_sha256} \
-          https://www.cp2k.org/static/downloads/scotch_${scotch_ver}.tar.gz
+        download_pkg_from_cp2k_org "${scotch_sha256}" "scotch_${scotch_ver}.tar.gz"
       fi
       echo "Installing from scratch into ${pkg_install_dir}"
       [ -d scotch_${scotch_ver} ] && rm -rf scotch_${scotch_ver}
@@ -61,7 +60,7 @@ case "${with_ptscotch}" in
       write_checksums "${install_lock_file}" "${SCRIPT_DIR}/stage5/$(basename ${SCRIPT_NAME})"
     fi
     SCOTCH_CFLAGS="-I'${pkg_install_dir}/include'"
-    SCOTCH_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    SCOTCH_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
   __SYSTEM__)
     echo "==================== Finding PT-Scotch from system paths ===================="
@@ -83,7 +82,7 @@ case "${with_ptscotch}" in
     check_dir "${pkg_install_dir}/lib"
     check_dir "${pkg_install_dir}/include"
     SCOTCH_CFLAGS="-I'${pkg_install_dir}/include'"
-    SCOTCH_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    SCOTCH_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
 esac
 if [ "$with_ptscotch" != "__DONTUSE__" ]; then
