@@ -30,8 +30,7 @@ case "$with_gsl" in
       if [ -f gsl-${gsl_ver}.tar.gz ]; then
         echo "gsl-${gsl_ver}.tar.gz is found"
       else
-        download_pkg ${DOWNLOADER_FLAGS} ${gls_sha256} \
-          "https://www.cp2k.org/static/downloads/gsl-${gsl_ver}.tar.gz"
+        download_pkg_from_cp2k_org "${gls_sha256}" "gsl-${gsl_ver}.tar.gz"
       fi
       echo "Installing from scratch into ${pkg_install_dir}"
       [ -d gsl-${gsl_ver} ] && rm -rf gsl-${gsl_ver}
@@ -49,7 +48,7 @@ case "$with_gsl" in
     fi
 
     GSL_CFLAGS="-I'${pkg_install_dir}/include'"
-    GSL_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    GSL_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
   __SYSTEM__)
     echo "==================== Finding gsl from system paths ===================="
@@ -66,7 +65,7 @@ case "$with_gsl" in
     check_dir "$pkg_install_dir/lib"
     check_dir "$pkg_install_dir/include"
     GSL_CFLAGS="-I'${pkg_install_dir}/include'"
-    GSL_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    GSL_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
 esac
 if [ "$with_gsl" != "__DONTUSE__" ]; then
