@@ -33,9 +33,7 @@ case "${with_libvori:=__INSTALL__}" in
       if [ -f libvori-${libvori_ver}.tar.gz ]; then
         echo "libvori-${libvori_ver}.tar.gz is found"
       else
-        # shellcheck disable=SC2086
-        download_pkg ${DOWNLOADER_FLAGS} ${libvori_sha256} \
-          "https://www.cp2k.org/static/downloads/libvori-${libvori_ver}.tar.gz"
+        download_pkg_from_cp2k_org "${libvori_sha256}" "libvori-${libvori_ver}.tar.gz"
       fi
 
       echo "Installing from scratch into ${pkg_install_dir}"
@@ -57,7 +55,7 @@ case "${with_libvori:=__INSTALL__}" in
 
       write_checksums "${install_lock_file}" "${SCRIPT_DIR}/stage7/$(basename "${SCRIPT_NAME}")"
     fi
-    LIBVORI_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    LIBVORI_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
   __SYSTEM__)
     echo "==================== Finding libvori from system paths ===================="
@@ -75,7 +73,7 @@ case "${with_libvori:=__INSTALL__}" in
     [ -d "${pkg_install_dir}/lib64" ] && LIBVORI_LIBDIR="${pkg_install_dir}/lib64"
 
     check_dir "${LIBVORI_LIBDIR}"
-    LIBVORI_LDFLAGS="-L'${LIBVORI_LIBDIR}' -Wl,-rpath='${LIBVORI_LIBDIR}'"
+    LIBVORI_LDFLAGS="-L'${LIBVORI_LIBDIR}' -Wl,-rpath,'${LIBVORI_LIBDIR}'"
     ;;
 esac
 
