@@ -30,8 +30,7 @@ case "$with_spglib" in
       if [ -f spglib-${spglib_ver}.tar.gz ]; then
         echo "spglib-${spglib_ver}.tar.gz is found"
       else
-        download_pkg ${DOWNLOADER_FLAGS} ${spglib_sha256} \
-          "https://www.cp2k.org/static/downloads/spglib-${spglib_ver}.tar.gz"
+        download_pkg_from_cp2k_org "${spglib_sha256}" "spglib-${spglib_ver}.tar.gz"
       fi
 
       echo "Installing from scratch into ${pkg_install_dir}"
@@ -55,7 +54,7 @@ case "$with_spglib" in
     fi
 
     SPGLIB_CFLAGS="-I${pkg_install_dir}/include"
-    SPGLIB_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    SPGLIB_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     if [ -d "${pkg_install_dir}/lib64" ]; then
       ln -sf lib64 ${pkg_install_dir}/lib
       cd ${pkg_install_dir}
@@ -75,7 +74,7 @@ case "$with_spglib" in
     check_dir "$pkg_install_dir/lib"
     check_dir "$pkg_install_dir/include"
     SPGLIB_CFLAGS="-I'${pkg_install_dir}/include'"
-    SPGLIB_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
+    SPGLIB_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
     ;;
 esac
 if [ "$with_spglib" != "__DONTUSE__" ]; then
