@@ -30,8 +30,8 @@ case "${with_spfft}" in
       if [ -f SpFFT-${spfft_ver}.tar.gz ]; then
         echo "SpFFT-${spfft_ver}.tar.gz is found"
       else
-        download_pkg ${DOWNLOADER_FLAGS} ${spfft_sha256} \
-          "https://www.cp2k.org/static/downloads/SpFFT-${spfft_ver}.tar.gz"
+        download_pkg_from_cp2k_org "${spfft_sha256}" "SpFFT-${spfft_ver}.tar.gz"
+
       fi
       if [ "${MATH_MODE}" = "mkl" ]; then
         EXTRA_CMAKE_FLAGS="-DSPFFT_MKL=ON -DSPFFT_FFTW_LIB=MKL"
@@ -112,7 +112,7 @@ case "${with_spfft}" in
             [ -f src/libspfft.a ] && install -m 644 src/*.a ${pkg_install_dir}/lib/cuda >> install.log 2>&1
             [ -f src/libspfft.so ] && install -m 644 src/*.so ${pkg_install_dir}/lib/cuda >> install.log 2>&1
             ;;
-          Mi50 | Mi100 | Mi200)
+          Mi50 | Mi100 | Mi200 | Mi250)
             [ -d build-hip ] && rm -rf "build-hip"
             mkdir build-hip
             cd build-hip
@@ -141,8 +141,8 @@ case "${with_spfft}" in
     fi
     SPFFT_ROOT="${pkg_install_dir}"
     SPFFT_CFLAGS="-I'${pkg_install_dir}/include'"
-    SPFFT_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
-    SPFFT_CUDA_LDFLAGS="-L'${pkg_install_dir}/lib/cuda' -Wl,-rpath='${pkg_install_dir}/lib/cuda'"
+    SPFFT_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
+    SPFFT_CUDA_LDFLAGS="-L'${pkg_install_dir}/lib/cuda' -Wl,-rpath,'${pkg_install_dir}/lib/cuda'"
     ;;
   __SYSTEM__)
     echo "==================== Finding spfft from system paths ===================="
@@ -164,7 +164,7 @@ case "${with_spfft}" in
     check_dir "${SPFFT_LIBDIR}"
     check_dir "${pkg_install_dir}/include"
     SPFFT_CFLAGS="-I'${pkg_install_dir}/include'"
-    SPFFT_LDFLAGS="-L'${SPFFT_LIBDIR}' -Wl,-rpath='${SPFFT_LIBDIR}'"
+    SPFFT_LDFLAGS="-L'${SPFFT_LIBDIR}' -Wl,-rpath,'${SPFFT_LIBDIR}'"
     ;;
 esac
 if [ "$with_spfft" != "__DONTUSE__" ]; then
