@@ -30,8 +30,8 @@ case "${with_spla}" in
       if [ -f SpLA-${spla_ver}.tar.gz ]; then
         echo "SpLA-${spla_ver}.tar.gz is found"
       else
-        download_pkg ${DOWNLOADER_FLAGS} ${spla_sha256} \
-          "https://www.cp2k.org/static/downloads/SpLA-${spla_ver}.tar.gz"
+        download_pkg_from_cp2k_org "${spla_sha256}" "SpLA-${spla_ver}.tar.gz"
+
       fi
       echo "Installing from scratch into ${pkg_install_dir}"
       [ -d SpLA-${spla_ver} ] && rm -rf SpLA-${spla_ver}
@@ -111,7 +111,7 @@ case "${with_spla}" in
             [ -f src/libspla.a ] && install -m 644 src/*.a ${pkg_install_dir}/lib/hip >> install.log 2>&1
             [ -f src/libspla.so ] && install -m 644 src/*.so ${pkg_install_dir}/lib/hip >> install.log 2>&1
             ;;
-          Mi50 | Mi100 | Mi200)
+          Mi50 | Mi100 | Mi200 | Mi250)
             [ -d build-hip ] && rm -rf "build-hip"
             mkdir build-hip
             cd build-hip
@@ -139,9 +139,9 @@ case "${with_spla}" in
     fi
     SPLA_ROOT="${pkg_install_dir}"
     SPLA_CFLAGS="-I'${pkg_install_dir}/include/spla'"
-    SPLA_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath='${pkg_install_dir}/lib'"
-    SPLA_CUDA_LDFLAGS="-L'${pkg_install_dir}/lib/cuda' -Wl,-rpath='${pkg_install_dir}/lib/cuda'"
-    SPLA_HIP_LDFLAGS="-L'${pkg_install_dir}/lib/hip' -Wl,-rpath='${pkg_install_dir}/lib/hip'"
+    SPLA_LDFLAGS="-L'${pkg_install_dir}/lib' -Wl,-rpath,'${pkg_install_dir}/lib'"
+    SPLA_CUDA_LDFLAGS="-L'${pkg_install_dir}/lib/cuda' -Wl,-rpath,'${pkg_install_dir}/lib/cuda'"
+    SPLA_HIP_LDFLAGS="-L'${pkg_install_dir}/lib/hip' -Wl,-rpath,'${pkg_install_dir}/lib/hip'"
     ;;
   __SYSTEM__)
     echo "==================== Finding spla from system paths ===================="
@@ -163,7 +163,7 @@ case "${with_spla}" in
     check_dir "${SPLA_LIBDIR}"
     check_dir "${pkg_install_dir}/include/spla"
     SPLA_CFLAGS="-I'${pkg_install_dir}/include/spla'"
-    SPLA_LDFLAGS="-L'${SPLA_LIBDIR}' -Wl,-rpath='${SPLA_LIBDIR}'"
+    SPLA_LDFLAGS="-L'${SPLA_LIBDIR}' -Wl,-rpath,'${SPLA_LIBDIR}'"
     ;;
 esac
 if [ "$with_spla" != "__DONTUSE__" ]; then
